@@ -1,13 +1,18 @@
 /*
- * Copyright (C) 2000-2018  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ *
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
 #include <config.h>
 
+#include <inttypes.h>
+#include <stdbool.h>
 #include <stdlib.h>
 #include <unistd.h>
 
@@ -50,18 +55,18 @@
 #endif
 #endif
 
-static isc_boolean_t short_form = ISC_TRUE,
-	tcpmode = ISC_FALSE, tcpmode_set = ISC_FALSE,
-	identify = ISC_FALSE, stats = ISC_TRUE,
-	comments = ISC_TRUE, section_question = ISC_TRUE,
-	section_answer = ISC_TRUE, section_authority = ISC_TRUE,
-	section_additional = ISC_TRUE, recurse = ISC_TRUE,
-	aaonly = ISC_FALSE, nofail = ISC_TRUE,
-	default_lookups = ISC_TRUE, a_noanswer = ISC_FALSE;
+static bool short_form = true,
+	tcpmode = false, tcpmode_set = false,
+	identify = false, stats = true,
+	comments = true, section_question = true,
+	section_answer = true, section_authority = true,
+	section_additional = true, recurse = true,
+	aaonly = false, nofail = true,
+	default_lookups = true, a_noanswer = false;
 
-static isc_boolean_t interactive;
+static bool interactive;
 
-static isc_boolean_t in_use = ISC_FALSE;
+static bool in_use = false;
 static char defclass[MXRD] = "IN";
 static char deftype[MXRD] = "A";
 static isc_event_t *global_event = NULL;
@@ -210,7 +215,7 @@ printrdata(dns_rdata_t *rdata) {
 	isc_result_t result;
 	isc_buffer_t *b = NULL;
 	unsigned int size = 1024;
-	isc_boolean_t done = ISC_FALSE;
+	bool done = false;
 
 	if (rdata->type < N_KNOWN_RRTYPES)
 		printf("%s", rtypetext[rdata->type]);
@@ -225,7 +230,7 @@ printrdata(dns_rdata_t *rdata) {
 		if (result == ISC_R_SUCCESS) {
 			printf("%.*s\n", (int)isc_buffer_usedlength(b),
 			       (char *)isc_buffer_base(b));
-			done = ISC_TRUE;
+			done = true;
 		} else if (result != ISC_R_NOSPACE)
 			check_result(result, "dns_rdata_totext");
 		isc_buffer_free(&b);
@@ -234,7 +239,7 @@ printrdata(dns_rdata_t *rdata) {
 }
 
 static isc_result_t
-printsection(dig_query_t *query, dns_message_t *msg, isc_boolean_t headers,
+printsection(dig_query_t *query, dns_message_t *msg, bool headers,
 	     dns_section_t section) {
 	isc_result_t result, loopresult;
 	dns_name_t *name;
@@ -301,7 +306,7 @@ printsection(dig_query_t *query, dns_message_t *msg, isc_boolean_t headers,
 }
 
 static isc_result_t
-detailsection(dig_query_t *query, dns_message_t *msg, isc_boolean_t headers,
+detailsection(dig_query_t *query, dns_message_t *msg, bool headers,
 	     dns_section_t section) {
 	isc_result_t result, loopresult;
 	dns_name_t *name;
@@ -426,7 +431,7 @@ chase_cnamechain(dns_message_t *msg, dns_name_t *qname) {
 }
 
 static isc_result_t
-printmessage(dig_query_t *query, dns_message_t *msg, isc_boolean_t headers) {
+printmessage(dig_query_t *query, dns_message_t *msg, bool headers) {
 	char servtext[ISC_SOCKADDR_FORMATSIZE];
 
 	/* I've we've gotten this far, we've reached a server. */
@@ -445,10 +450,10 @@ printmessage(dig_query_t *query, dns_message_t *msg, isc_boolean_t headers) {
 	if (!short_form) {
 		puts("------------");
 		/*		detailheader(query, msg);*/
-		detailsection(query, msg, ISC_TRUE, DNS_SECTION_QUESTION);
-		detailsection(query, msg, ISC_TRUE, DNS_SECTION_ANSWER);
-		detailsection(query, msg, ISC_TRUE, DNS_SECTION_AUTHORITY);
-		detailsection(query, msg, ISC_TRUE, DNS_SECTION_ADDITIONAL);
+		detailsection(query, msg, true, DNS_SECTION_QUESTION);
+		detailsection(query, msg, true, DNS_SECTION_ANSWER);
+		detailsection(query, msg, true, DNS_SECTION_AUTHORITY);
+		detailsection(query, msg, true, DNS_SECTION_ADDITIONAL);
 		puts("------------");
 	}
 
@@ -472,17 +477,16 @@ printmessage(dig_query_t *query, dns_message_t *msg, isc_boolean_t headers) {
 		dns_name_t *name;
 
 		/* Add AAAA lookup. */
-		dns_fixedname_init(&fixed);
-		name = dns_fixedname_name(&fixed);
+		name = dns_fixedname_initname(&fixed);
 		dns_name_copy(query->lookup->name, name, NULL);
 		chase_cnamechain(msg, name);
 		dns_name_format(name, namestr, sizeof(namestr));
-		lookup = clone_lookup(query->lookup, ISC_FALSE);
+		lookup = clone_lookup(query->lookup, false);
 		if (lookup != NULL) {
 			strlcpy(lookup->textname, namestr,
 				sizeof(lookup->textname));
 			lookup->rdtype = dns_rdatatype_aaaa;
-			lookup->rdtypeset = ISC_TRUE;
+			lookup->rdtypeset = true;
 			lookup->origin = NULL;
 			lookup->retries = tries;
 			ISC_LIST_APPEND(lookup_list, lookup, link);
@@ -496,7 +500,7 @@ printmessage(dig_query_t *query, dns_message_t *msg, isc_boolean_t headers) {
 		printsection(query, msg, headers, DNS_SECTION_ANSWER);
 	else {
 		if (default_lookups && query->lookup->rdtype == dns_rdatatype_a)
-			a_noanswer = ISC_TRUE;
+			a_noanswer = true;
 
 		else if (!default_lookups ||
 			 (query->lookup->rdtype == dns_rdatatype_aaaa &&
@@ -518,7 +522,7 @@ printmessage(dig_query_t *query, dns_message_t *msg, isc_boolean_t headers) {
 }
 
 static void
-show_settings(isc_boolean_t full, isc_boolean_t serv_only) {
+show_settings(bool full, bool serv_only) {
 	dig_server_t *srv;
 	isc_sockaddr_t sockaddr;
 	dig_searchlist_t *listent;
@@ -563,7 +567,7 @@ show_settings(isc_boolean_t full, isc_boolean_t serv_only) {
 	printf("\n");
 }
 
-static isc_boolean_t
+static bool
 testtype(char *typetext) {
 	isc_result_t result;
 	isc_textregion_t tr;
@@ -573,14 +577,14 @@ testtype(char *typetext) {
 	tr.length = strlen(typetext);
 	result = dns_rdatatype_fromtext(&rdtype, &tr);
 	if (result == ISC_R_SUCCESS)
-		return (ISC_TRUE);
+		return (true);
 	else {
 		printf("unknown query type: %s\n", typetext);
-		return (ISC_FALSE);
+		return (false);
 	}
 }
 
-static isc_boolean_t
+static bool
 testclass(char *typetext) {
 	isc_result_t result;
 	isc_textregion_t tr;
@@ -590,24 +594,24 @@ testclass(char *typetext) {
 	tr.length = strlen(typetext);
 	result = dns_rdataclass_fromtext(&rdclass, &tr);
 	if (result == ISC_R_SUCCESS)
-		return (ISC_TRUE);
+		return (true);
 	else {
 		printf("unknown query class: %s\n", typetext);
-		return (ISC_FALSE);
+		return (false);
 	}
 }
 
 static void
 set_port(const char *value) {
-	isc_uint32_t n;
+	uint32_t n;
 	isc_result_t result = parse_uint(&n, value, 65535, "port");
 	if (result == ISC_R_SUCCESS)
-		port = (isc_uint16_t) n;
+		port = (uint16_t) n;
 }
 
 static void
 set_timeout(const char *value) {
-	isc_uint32_t n;
+	uint32_t n;
 	isc_result_t result = parse_uint(&n, value, UINT_MAX, "timeout");
 	if (result == ISC_R_SUCCESS)
 		timeout = n;
@@ -615,7 +619,7 @@ set_timeout(const char *value) {
 
 static void
 set_tries(const char *value) {
-	isc_uint32_t n;
+	uint32_t n;
 	isc_result_t result = parse_uint(&n, value, INT_MAX, "tries");
 	if (result == ISC_R_SUCCESS)
 		tries = n;
@@ -623,7 +627,7 @@ set_tries(const char *value) {
 
 static void
 set_ndots(const char *value) {
-	isc_uint32_t n;
+	uint32_t n;
 	isc_result_t result = parse_uint(&n, value, 128, "ndots");
 	if (result == ISC_R_SUCCESS)
 		ndots = n;
@@ -642,7 +646,7 @@ setoption(char *opt) {
 	((l >= N) && (l < sizeof(A)) && (strncasecmp(opt, A, l) == 0))
 
 	if (CHECKOPT("all", 3)) {
-		show_settings(ISC_TRUE, ISC_FALSE);
+		show_settings(true, false);
 	} else if (strncasecmp(opt, "class=", 6) == 0) {
 		if (testclass(&opt[6]))
 			strlcpy(defclass, &opt[6], sizeof(defclass));
@@ -652,41 +656,41 @@ setoption(char *opt) {
 	} else if (strncasecmp(opt, "type=", 5) == 0) {
 		if (testtype(&opt[5])) {
 			strlcpy(deftype, &opt[5], sizeof(deftype));
-			default_lookups = ISC_FALSE;
+			default_lookups = false;
 		}
 	} else if (strncasecmp(opt, "ty=", 3) == 0) {
 		if (testtype(&opt[3])) {
 			strlcpy(deftype, &opt[3], sizeof(deftype));
-			default_lookups = ISC_FALSE;
+			default_lookups = false;
 		}
 	} else if (strncasecmp(opt, "querytype=", 10) == 0) {
 		if (testtype(&opt[10])) {
 			strlcpy(deftype, &opt[10], sizeof(deftype));
-			default_lookups = ISC_FALSE;
+			default_lookups = false;
 		}
 	} else if (strncasecmp(opt, "query=", 6) == 0) {
 		if (testtype(&opt[6])) {
 			strlcpy(deftype, &opt[6], sizeof(deftype));
-			default_lookups = ISC_FALSE;
+			default_lookups = false;
 		}
 	} else if (strncasecmp(opt, "qu=", 3) == 0) {
 		if (testtype(&opt[3])) {
 			strlcpy(deftype, &opt[3], sizeof(deftype));
-			default_lookups = ISC_FALSE;
+			default_lookups = false;
 		}
 	} else if (strncasecmp(opt, "q=", 2) == 0) {
 		if (testtype(&opt[2])) {
 			strlcpy(deftype, &opt[2], sizeof(deftype));
-			default_lookups = ISC_FALSE;
+			default_lookups = false;
 		}
 	} else if (strncasecmp(opt, "domain=", 7) == 0) {
 		strlcpy(domainopt, &opt[7], sizeof(domainopt));
 		set_search_domain(domainopt);
-		usesearch = ISC_TRUE;
+		usesearch = true;
 	} else if (strncasecmp(opt, "do=", 3) == 0) {
 		strlcpy(domainopt, &opt[3], sizeof(domainopt));
 		set_search_domain(domainopt);
-		usesearch = ISC_TRUE;
+		usesearch = true;
 	} else if (strncasecmp(opt, "port=", 5) == 0) {
 		set_port(&opt[5]);
 	} else if (strncasecmp(opt, "po=", 3) == 0) {
@@ -696,43 +700,43 @@ setoption(char *opt) {
 	} else if (strncasecmp(opt, "t=", 2) == 0) {
 		set_timeout(&opt[2]);
 	} else if (CHECKOPT("recurse", 3)) {
-		recurse = ISC_TRUE;
+		recurse = true;
 	} else if (CHECKOPT("norecurse", 5)) {
-		recurse = ISC_FALSE;
+		recurse = false;
 	} else if (strncasecmp(opt, "retry=", 6) == 0) {
 		set_tries(&opt[6]);
 	} else if (strncasecmp(opt, "ret=", 4) == 0) {
 		set_tries(&opt[4]);
 	} else if (CHECKOPT("defname", 3)) {
-		usesearch = ISC_TRUE;
+		usesearch = true;
 	} else if (CHECKOPT("nodefname", 5)) {
-		usesearch = ISC_FALSE;
-	} else if (CHECKOPT("vc", 2) == 0) {
-		tcpmode = ISC_TRUE;
-		tcpmode_set = ISC_TRUE;
-	} else if (CHECKOPT("novc", 4) == 0) {
-		tcpmode = ISC_FALSE;
-		tcpmode_set = ISC_TRUE;
-	} else if (CHECKOPT("debug", 3) == 0) {
-		short_form = ISC_FALSE;
-		showsearch = ISC_TRUE;
-	} else if (CHECKOPT("nodebug", 5) == 0) {
-		short_form = ISC_TRUE;
-		showsearch = ISC_FALSE;
-	} else if (CHECKOPT("d2", 2) == 0) {
-		debugging = ISC_TRUE;
-	} else if (CHECKOPT("nod2", 4) == 0) {
-		debugging = ISC_FALSE;
-	} else if (CHECKOPT("search", 3) == 0) {
-		usesearch = ISC_TRUE;
-	} else if (CHECKOPT("nosearch", 5) == 0) {
-		usesearch = ISC_FALSE;
-	} else if (CHECKOPT("sil", 3) == 0) {
-		/* deprecation_msg = ISC_FALSE; */
-	} else if (CHECKOPT("fail", 3) == 0) {
-		nofail=ISC_FALSE;
-	} else if (CHECKOPT("nofail", 5) == 0) {
-		nofail=ISC_TRUE;
+		usesearch = false;
+	} else if (CHECKOPT("vc", 2)) {
+		tcpmode = true;
+		tcpmode_set = true;
+	} else if (CHECKOPT("novc", 4)) {
+		tcpmode = false;
+		tcpmode_set = true;
+	} else if (CHECKOPT("debug", 3)) {
+		short_form = false;
+		showsearch = true;
+	} else if (CHECKOPT("nodebug", 5)) {
+		short_form = true;
+		showsearch = false;
+	} else if (CHECKOPT("d2", 2)) {
+		debugging = true;
+	} else if (CHECKOPT("nod2", 4)) {
+		debugging = false;
+	} else if (CHECKOPT("search", 3)) {
+		usesearch = true;
+	} else if (CHECKOPT("nosearch", 5)) {
+		usesearch = false;
+	} else if (CHECKOPT("sil", 3)) {
+		/* deprecation_msg = false; */
+	} else if (CHECKOPT("fail", 3)) {
+		nofail=false;
+	} else if (CHECKOPT("nofail", 5)) {
+		nofail=true;
 	} else if (strncasecmp(opt, "ndots=", 6) == 0) {
 		set_ndots(&opt[6]);
 	} else {
@@ -751,7 +755,7 @@ addlookup(char *opt) {
 
 	debug("addlookup()");
 
-	a_noanswer = ISC_FALSE;
+	a_noanswer = false;
 
 	tr.base = deftype;
 	tr.length = strlen(deftype);
@@ -768,21 +772,21 @@ addlookup(char *opt) {
 		rdclass = dns_rdataclass_in;
 	}
 	lookup = make_empty_lookup();
-	if (get_reverse(store, sizeof(store), opt, lookup->ip6_int, ISC_TRUE)
+	if (get_reverse(store, sizeof(store), opt, lookup->ip6_int, true)
 	    == ISC_R_SUCCESS) {
 		strlcpy(lookup->textname, store, sizeof(lookup->textname));
 		lookup->rdtype = dns_rdatatype_ptr;
-		lookup->rdtypeset = ISC_TRUE;
+		lookup->rdtypeset = true;
 	} else {
 		strlcpy(lookup->textname, opt, sizeof(lookup->textname));
 		lookup->rdtype = rdtype;
-		lookup->rdtypeset = ISC_TRUE;
+		lookup->rdtypeset = true;
 	}
 	lookup->rdclass = rdclass;
-	lookup->rdclassset = ISC_TRUE;
-	lookup->trace = ISC_FALSE;
+	lookup->rdclassset = true;
+	lookup->trace = false;
 	lookup->trace_root = lookup->trace;
-	lookup->ns_search_only = ISC_FALSE;
+	lookup->ns_search_only = false;
 	lookup->identify = identify;
 	lookup->recurse = recurse;
 	lookup->aaonly = aaonly;
@@ -790,7 +794,7 @@ addlookup(char *opt) {
 	lookup->udpsize = 0;
 	lookup->comments = comments;
 	if (lookup->rdtype == dns_rdatatype_any && !tcpmode_set)
-		lookup->tcp_mode = ISC_TRUE;
+		lookup->tcp_mode = true;
 	else
 		lookup->tcp_mode = tcpmode;
 	lookup->stats = stats;
@@ -798,9 +802,9 @@ addlookup(char *opt) {
 	lookup->section_answer = section_answer;
 	lookup->section_authority = section_authority;
 	lookup->section_additional = section_additional;
-	lookup->new_search = ISC_TRUE;
+	lookup->new_search = true;
 	if (nofail)
-		lookup->servfail_stops = ISC_FALSE;
+		lookup->servfail_stops = false;
 	ISC_LIST_INIT(lookup->q);
 	ISC_LINK_INIT(lookup, link);
 	ISC_LIST_APPEND(lookup_list, lookup, link);
@@ -824,11 +828,11 @@ do_next_command(char *input) {
 		 (strcasecmp(ptr, "lserver") == 0)) {
 		isc_app_block();
 		set_nameserver(arg);
-		check_ra = ISC_FALSE;
+		check_ra = false;
 		isc_app_unblock();
-		show_settings(ISC_TRUE, ISC_TRUE);
+		show_settings(true, true);
 	} else if (strcasecmp(ptr, "exit") == 0) {
-		in_use = ISC_FALSE;
+		in_use = false;
 	} else if (strcasecmp(ptr, "help") == 0 ||
 		   strcasecmp(ptr, "?") == 0) {
 		printf("The '%s' command is not yet implemented.\n", ptr);
@@ -865,7 +869,7 @@ get_next_command(void) {
 		ptr = fgets(buf, COMMSIZE, stdin);
 	isc_app_unblock();
 	if (ptr == NULL) {
-		in_use = ISC_FALSE;
+		in_use = false;
 	} else
 		do_next_command(ptr);
 #ifdef HAVE_READLINE
@@ -875,12 +879,29 @@ get_next_command(void) {
 	isc_mem_free(mctx, buf);
 }
 
+ISC_PLATFORM_NORETURN_PRE static void
+usage(void) ISC_PLATFORM_NORETURN_POST;
+
+static void
+usage(void) {
+    fprintf(stderr, "Usage:\n");
+    fprintf(stderr,
+"   nslookup [-opt ...]             # interactive mode using default server\n");
+    fprintf(stderr,
+"   nslookup [-opt ...] - server    # interactive mode using 'server'\n");
+    fprintf(stderr,
+"   nslookup [-opt ...] host        # just look up 'host' using default server\n");
+    fprintf(stderr,
+"   nslookup [-opt ...] host server # just look up 'host' using 'server'\n");
+    exit(1);
+}
+
 static void
 parse_args(int argc, char **argv) {
-	isc_boolean_t have_lookup = ISC_FALSE;
+	bool have_lookup = false;
 
-	usesearch = ISC_TRUE;
-	for (argc--, argv++; argc > 0; argc--, argv++) {
+	usesearch = true;
+	for (argc--, argv++; argc > 0 && argv[0] != NULL; argc--, argv++) {
 		debug("main parsing %s", argv[0]);
 		if (argv[0][0] == '-') {
 			if (strncasecmp(argv[0], "-ver", 4) == 0) {
@@ -889,15 +910,18 @@ parse_args(int argc, char **argv) {
 			} else if (argv[0][1] != 0) {
 				setoption(&argv[0][1]);
 			} else
-				have_lookup = ISC_TRUE;
+				have_lookup = true;
 		} else {
 			if (!have_lookup) {
-				have_lookup = ISC_TRUE;
-				in_use = ISC_TRUE;
+				have_lookup = true;
+				in_use = true;
 				addlookup(argv[0]);
 			} else {
+				if (argv[1] != NULL) {
+					usage();
+				}
 				set_nameserver(argv[0]);
-				check_ra = ISC_FALSE;
+				check_ra = false;
 			}
 		}
 	}
@@ -968,13 +992,13 @@ int
 main(int argc, char **argv) {
 	isc_result_t result;
 
-	interactive = ISC_TF(isatty(0));
+	interactive = isatty(0);
 
 	ISC_LIST_INIT(lookup_list);
 	ISC_LIST_INIT(server_list);
 	ISC_LIST_INIT(search_list);
 
-	check_ra = ISC_TRUE;
+	check_ra = true;
 
 	/* setup dighost callbacks */
 	dighost_printmessage = printmessage;
@@ -988,7 +1012,7 @@ main(int argc, char **argv) {
 	setup_libs();
 	progname = argv[0];
 
-	setup_system(ISC_FALSE, ISC_FALSE);
+	setup_system(false, false);
 	parse_args(argc, argv);
 	if (keyfile[0] != 0)
 		setup_file_key();
@@ -1002,7 +1026,7 @@ main(int argc, char **argv) {
 	else
 		result = isc_app_onrun(mctx, global_task, getinput, NULL);
 	check_result(result, "isc_app_onrun");
-	in_use = ISC_TF(!in_use);
+	in_use = !in_use;
 
 	(void)isc_app_run();
 
