@@ -1,26 +1,20 @@
 /*
- * Copyright (C) 2004-2009, 2011, 2013, 2014  Internet Systems Consortium, Inc. ("ISC")
- * Copyright (C) 1999-2002  Internet Software Consortium.
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
-/* $Id: acl.c,v 1.55 2011/06/17 23:47:49 tbox Exp $ */
-
 /*! \file */
 
 #include <config.h>
 
+#include <stdbool.h>
+
 #include <isc/mem.h>
 #include <isc/once.h>
 #include <isc/string.h>
@@ -29,6 +23,7 @@
 #include <dns/acl.h>
 #include <dns/iptable.h>
 
+
 /*
  * Create a new ACL, including an IP table and an array with room
  * for 'n' ACL elements.  The elements are uninitialized and the
@@ -69,7 +64,7 @@ dns_acl_create(isc_mem_t *mctx, int n, dns_acl_t **target) {
 	acl->elements = NULL;
 	acl->alloc = 0;
 	acl->length = 0;
-	acl->has_negatives = ISC_FALSE;
+	acl->has_negatives = false;
 
 	ISC_LINK_INIT(acl, nextincache);
 	/*
@@ -99,7 +94,7 @@ dns_acl_create(isc_mem_t *mctx, int n, dns_acl_t **target) {
  * "none" is the same as "!any".
  */
 static isc_result_t
-dns_acl_anyornone(isc_mem_t *mctx, isc_boolean_t neg, dns_acl_t **target) {
+dns_acl_anyornone(isc_mem_t *mctx, bool neg, dns_acl_t **target) {
 	isc_result_t result;
 	dns_acl_t *acl = NULL;
 
@@ -107,7 +102,7 @@ dns_acl_anyornone(isc_mem_t *mctx, isc_boolean_t neg, dns_acl_t **target) {
 	if (result != ISC_R_SUCCESS)
 		return (result);
 
-	result = dns_iptable_addprefix(acl->iptable, NULL, 0, ISC_TF(!neg));
+	result = dns_iptable_addprefix(acl->iptable, NULL, 0, !neg);
 	if (result != ISC_R_SUCCESS) {
 		dns_acl_detach(&acl);
 		return (result);
@@ -122,7 +117,7 @@ dns_acl_anyornone(isc_mem_t *mctx, isc_boolean_t neg, dns_acl_t **target) {
  */
 isc_result_t
 dns_acl_any(isc_mem_t *mctx, dns_acl_t **target) {
-	return (dns_acl_anyornone(mctx, ISC_FALSE, target));
+	return (dns_acl_anyornone(mctx, false, target));
 }
 
 /*
@@ -130,15 +125,15 @@ dns_acl_any(isc_mem_t *mctx, dns_acl_t **target) {
  */
 isc_result_t
 dns_acl_none(isc_mem_t *mctx, dns_acl_t **target) {
-	return (dns_acl_anyornone(mctx, ISC_TRUE, target));
+	return (dns_acl_anyornone(mctx, true, target));
 }
 
 /*
- * If pos is ISC_TRUE, test whether acl is set to "{ any; }"
- * If pos is ISC_FALSE, test whether acl is set to "{ none; }"
+ * If pos is true, test whether acl is set to "{ any; }"
+ * If pos is false, test whether acl is set to "{ none; }"
  */
-static isc_boolean_t
-dns_acl_isanyornone(dns_acl_t *acl, isc_boolean_t pos)
+static bool
+dns_acl_isanyornone(dns_acl_t *acl, bool pos)
 {
 	/* Should never happen but let's be safe */
 	if (acl == NULL ||
@@ -146,37 +141,37 @@ dns_acl_isanyornone(dns_acl_t *acl, isc_boolean_t pos)
 	    acl->iptable->radix == NULL ||
 	    acl->iptable->radix->head == NULL ||
 	    acl->iptable->radix->head->prefix == NULL)
-		return (ISC_FALSE);
+		return (false);
 
 	if (acl->length != 0 || acl->node_count != 1)
-		return (ISC_FALSE);
+		return (false);
 
 	if (acl->iptable->radix->head->prefix->bitlen == 0 &&
 	    acl->iptable->radix->head->data[0] != NULL &&
 	    acl->iptable->radix->head->data[0] ==
 		    acl->iptable->radix->head->data[1] &&
-	    *(isc_boolean_t *) (acl->iptable->radix->head->data[0]) == pos)
-		return (ISC_TRUE);
+	    *(bool *) (acl->iptable->radix->head->data[0]) == pos)
+		return (true);
 
-	return (ISC_FALSE); /* All others */
+	return (false); /* All others */
 }
 
 /*
  * Test whether acl is set to "{ any; }"
  */
-isc_boolean_t
+bool
 dns_acl_isany(dns_acl_t *acl)
 {
-	return (dns_acl_isanyornone(acl, ISC_TRUE));
+	return (dns_acl_isanyornone(acl, true));
 }
 
 /*
  * Test whether acl is set to "{ none; }"
  */
-isc_boolean_t
+bool
 dns_acl_isnone(dns_acl_t *acl)
 {
-	return (dns_acl_isanyornone(acl, ISC_FALSE));
+	return (dns_acl_isanyornone(acl, false));
 }
 
 /*
@@ -193,10 +188,25 @@ dns_acl_match(const isc_netaddr_t *reqaddr,
 	      int *match,
 	      const dns_aclelement_t **matchelt)
 {
-	isc_uint16_t bitlen, family;
+	return (dns_acl_match2(reqaddr, reqsigner, NULL, 0, NULL, acl, env,
+			       match, matchelt));
+}
+
+isc_result_t
+dns_acl_match2(const isc_netaddr_t *reqaddr,
+	       const dns_name_t *reqsigner,
+	       const isc_netaddr_t *ecs,
+	       uint8_t ecslen,
+	       uint8_t *scope,
+	       const dns_acl_t *acl,
+	       const dns_aclenv_t *env,
+	       int *match,
+	       const dns_aclelement_t **matchelt)
+{
+	uint16_t bitlen;
 	isc_prefix_t pfx;
 	isc_radix_node_t *node = NULL;
-	const isc_netaddr_t *addr;
+	const isc_netaddr_t *addr = reqaddr;
 	isc_netaddr_t v4addr;
 	isc_result_t result;
 	int match_num = -1;
@@ -204,20 +214,19 @@ dns_acl_match(const isc_netaddr_t *reqaddr,
 
 	REQUIRE(reqaddr != NULL);
 	REQUIRE(matchelt == NULL || *matchelt == NULL);
+	REQUIRE(ecs != NULL || scope == NULL);
 
-	if (env == NULL || env->match_mapped == ISC_FALSE ||
-	    reqaddr->family != AF_INET6 ||
-	    !IN6_IS_ADDR_V4MAPPED(&reqaddr->type.in6))
-		addr = reqaddr;
-	else {
-		isc_netaddr_fromv4mapped(&v4addr, reqaddr);
+	if (env != NULL && env->match_mapped &&
+	    addr->family == AF_INET6 &&
+	    IN6_IS_ADDR_V4MAPPED(&addr->type.in6))
+	{
+		isc_netaddr_fromv4mapped(&v4addr, addr);
 		addr = &v4addr;
 	}
 
 	/* Always match with host addresses. */
-	family = addr->family;
-	bitlen = family == AF_INET6 ? 128 : 32;
-	NETADDR_TO_PREFIX_T(addr, pfx, bitlen);
+	bitlen = (addr->family == AF_INET6) ? 128 : 32;
+	NETADDR_TO_PREFIX_T(addr, pfx, bitlen, false);
 
 	/* Assume no match. */
 	*match = 0;
@@ -227,11 +236,54 @@ dns_acl_match(const isc_netaddr_t *reqaddr,
 
 	/* Found a match. */
 	if (result == ISC_R_SUCCESS && node != NULL) {
-		match_num = node->node_num[ISC_IS6(family)];
-		if (*(isc_boolean_t *) node->data[ISC_IS6(family)] == ISC_TRUE)
+		int fam = ISC_RADIX_FAMILY(&pfx);
+		match_num = node->node_num[fam];
+		if (*(bool *) node->data[fam]) {
 			*match = match_num;
-		else
+		} else {
 			*match = -match_num;
+		}
+	}
+
+	isc_refcount_destroy(&pfx.refcount);
+
+	/*
+	 * If ecs is not NULL, we search the radix tree again to
+	 * see if we find a better match on an ECS node
+	 */
+	if (ecs != NULL) {
+		node = NULL;
+		addr = ecs;
+
+		if (env != NULL && env->match_mapped &&
+		    addr->family == AF_INET6 &&
+		    IN6_IS_ADDR_V4MAPPED(&addr->type.in6))
+		{
+			isc_netaddr_fromv4mapped(&v4addr, addr);
+			addr = &v4addr;
+		}
+
+		NETADDR_TO_PREFIX_T(addr, pfx, ecslen, true);
+
+		result = isc_radix_search(acl->iptable->radix, &node, &pfx);
+		if (result == ISC_R_SUCCESS && node != NULL) {
+			int off = ISC_RADIX_FAMILY(&pfx);
+			if (match_num == -1 ||
+			    node->node_num[off] < match_num)
+			{
+				match_num = node->node_num[off];
+				if (scope != NULL) {
+					*scope = node->bit;
+				}
+				if (*(bool *) node->data[off]) {
+					*match = match_num;
+				} else {
+					*match = -match_num;
+				}
+			}
+		}
+
+		isc_refcount_destroy(&pfx.refcount);
 	}
 
 	/* Now search non-radix elements for a match with a lower node_num. */
@@ -240,24 +292,22 @@ dns_acl_match(const isc_netaddr_t *reqaddr,
 
 		/* Already found a better match? */
 		if (match_num != -1 && match_num < e->node_num) {
-			isc_refcount_destroy(&pfx.refcount);
-			return (ISC_R_SUCCESS);
+			break;
 		}
 
-		if (dns_aclelement_match(reqaddr, reqsigner,
-					 e, env, matchelt)) {
+		if (dns_aclelement_match2(reqaddr, reqsigner, ecs, ecslen,
+					  scope, e, env, matchelt))
+		{
 			if (match_num == -1 || e->node_num < match_num) {
-				if (e->negative == ISC_TRUE)
+				if (e->negative)
 					*match = -e->node_num;
 				else
 					*match = e->node_num;
 			}
-			isc_refcount_destroy(&pfx.refcount);
-			return (ISC_R_SUCCESS);
+			break;
 		}
 	}
 
-	isc_refcount_destroy(&pfx.refcount);
 	return (ISC_R_SUCCESS);
 }
 
@@ -271,7 +321,7 @@ dns_acl_match(const isc_netaddr_t *reqaddr,
  * an unexpected positive match in the parent ACL.
  */
 isc_result_t
-dns_acl_merge(dns_acl_t *dest, dns_acl_t *source, isc_boolean_t pos)
+dns_acl_merge(dns_acl_t *dest, dns_acl_t *source, bool pos)
 {
 	isc_result_t result;
 	unsigned int newalloc, nelem, i;
@@ -339,9 +389,17 @@ dns_acl_merge(dns_acl_t *dest, dns_acl_t *source, isc_boolean_t pos)
 				return result;
 		}
 
+#ifdef HAVE_GEOIP
+		/* Duplicate GeoIP data */
+		if (source->elements[i].type == dns_aclelementtype_geoip) {
+			dest->elements[nelem + i].geoip_elem =
+				source->elements[i].geoip_elem;
+		}
+#endif
+
 		/* reverse sense of positives if this is a negative acl */
-		if (!pos && source->elements[i].negative == ISC_FALSE) {
-			dest->elements[nelem + i].negative = ISC_TRUE;
+		if (!pos && !source->elements[i].negative) {
+			dest->elements[nelem + i].negative = true;
 		} else {
 			dest->elements[nelem + i].negative =
 				source->elements[i].negative;
@@ -364,23 +422,42 @@ dns_acl_merge(dns_acl_t *dest, dns_acl_t *source, isc_boolean_t pos)
 
 /*
  * Like dns_acl_match, but matches against the single ACL element 'e'
- * rather than a complete ACL, and returns ISC_TRUE iff it matched.
+ * rather than a complete ACL, and returns true iff it matched.
  *
  * To determine whether the match was positive or negative, the
  * caller should examine e->negative.  Since the element 'e' may be
  * a reference to a named ACL or a nested ACL, a matching element
  * returned through 'matchelt' is not necessarily 'e' itself.
  */
-isc_boolean_t
+bool
 dns_aclelement_match(const isc_netaddr_t *reqaddr,
 		     const dns_name_t *reqsigner,
 		     const dns_aclelement_t *e,
 		     const dns_aclenv_t *env,
 		     const dns_aclelement_t **matchelt)
+{
+	return (dns_aclelement_match2(reqaddr, reqsigner, NULL, 0, NULL,
+				      e, env, matchelt));
+}
+
+bool
+dns_aclelement_match2(const isc_netaddr_t *reqaddr,
+		      const dns_name_t *reqsigner,
+		      const isc_netaddr_t *ecs,
+		      uint8_t ecslen,
+		      uint8_t *scope,
+		      const dns_aclelement_t *e,
+		      const dns_aclenv_t *env,
+		      const dns_aclelement_t **matchelt)
 {
 	dns_acl_t *inner = NULL;
 	int indirectmatch;
 	isc_result_t result;
+#ifdef HAVE_GEOIP
+	const isc_netaddr_t *addr = NULL;
+#endif
+
+	REQUIRE(ecs != NULL || scope == NULL);
 
 	switch (e->type) {
 	case dns_aclelementtype_keyname:
@@ -388,10 +465,9 @@ dns_aclelement_match(const isc_netaddr_t *reqaddr,
 		    dns_name_equal(reqsigner, &e->keyname)) {
 			if (matchelt != NULL)
 				*matchelt = e;
-			return (ISC_TRUE);
-		} else {
-			return (ISC_FALSE);
-		}
+			return (true);
+		} else
+			return (false);
 
 	case dns_aclelementtype_nestedacl:
 		inner = e->nestedacl;
@@ -399,23 +475,31 @@ dns_aclelement_match(const isc_netaddr_t *reqaddr,
 
 	case dns_aclelementtype_localhost:
 		if (env == NULL || env->localhost == NULL)
-			return (ISC_FALSE);
+			return (false);
 		inner = env->localhost;
 		break;
 
 	case dns_aclelementtype_localnets:
 		if (env == NULL || env->localnets == NULL)
-			return (ISC_FALSE);
+			return (false);
 		inner = env->localnets;
 		break;
 
+#ifdef HAVE_GEOIP
+	case dns_aclelementtype_geoip:
+		if (env == NULL || env->geoip == NULL)
+			return (false);
+		addr = (env->geoip_use_ecs && ecs != NULL) ? ecs : reqaddr;
+		return (dns_geoip_match(addr, scope, env->geoip,
+					&e->geoip_elem));
+#endif
 	default:
-		/* Should be impossible. */
 		INSIST(0);
+		ISC_UNREACHABLE();
 	}
 
-	result = dns_acl_match(reqaddr, reqsigner, inner, env,
-			       &indirectmatch, matchelt);
+	result = dns_acl_match2(reqaddr, reqsigner, ecs, ecslen, scope,
+				inner, env, &indirectmatch, matchelt);
 	INSIST(result == ISC_R_SUCCESS);
 
 	/*
@@ -424,22 +508,20 @@ dns_aclelement_match(const isc_netaddr_t *reqaddr,
 	 * surprise positive match through double negation.
 	 * XXXDCL this should be documented.
 	 */
-
 	if (indirectmatch > 0) {
 		if (matchelt != NULL)
 			*matchelt = e;
-		return (ISC_TRUE);
+		return (true);
 	}
 
 	/*
 	 * A negative indirect match may have set *matchelt, but we don't
 	 * want it set when we return.
 	 */
-
 	if (matchelt != NULL)
 		*matchelt = NULL;
 
-	return (ISC_FALSE);
+	return (false);
 }
 
 void
@@ -492,7 +574,7 @@ dns_acl_detach(dns_acl_t **aclp) {
 
 static isc_once_t	insecure_prefix_once = ISC_ONCE_INIT;
 static isc_mutex_t	insecure_prefix_lock;
-static isc_boolean_t	insecure_prefix_found;
+static bool	insecure_prefix_found;
 
 static void
 initialize_action(void) {
@@ -500,55 +582,55 @@ initialize_action(void) {
 }
 
 /*
- * Called via isc_radix_walk() to find IP table nodes that are
+ * Called via isc_radix_process() to find IP table nodes that are
  * insecure.
  */
 static void
 is_insecure(isc_prefix_t *prefix, void **data) {
-	isc_boolean_t secure;
-	int bitlen, family;
-
-	bitlen = prefix->bitlen;
-	family = prefix->family;
-
-	/* Negated entries are always secure. */
-	secure = * (isc_boolean_t *)data[ISC_IS6(family)];
-	if (!secure) {
+	/*
+	 * If all nonexistent or negative then this node is secure.
+	 */
+	if ((data[0] == NULL || !* (bool *) data[0]) &&
+	    (data[1] == NULL || !* (bool *) data[1]) &&
+	    (data[2] == NULL || !* (bool *) data[2]) &&
+	    (data[3] == NULL || !* (bool *) data[3]))
 		return;
-	}
 
-	/* If loopback prefix found, return */
-	switch (family) {
-	case AF_INET:
-		if (bitlen == 32 &&
-		    htonl(prefix->add.sin.s_addr) == INADDR_LOOPBACK)
-			return;
-		break;
-	case AF_INET6:
-		if (bitlen == 128 && IN6_IS_ADDR_LOOPBACK(&prefix->add.sin6))
-			return;
-		break;
-	default:
-		break;
-	}
+	/*
+	 * If a loopback address found and the other family and
+	 * ecs entry doesn't exist or is negative, return.
+	 */
+	if (prefix->bitlen == 32 &&
+	    htonl(prefix->add.sin.s_addr) == INADDR_LOOPBACK &&
+	    (data[1] == NULL || !* (bool *) data[1]) &&
+	    (data[2] == NULL || !* (bool *) data[2]) &&
+	    (data[3] == NULL || !* (bool *) data[3]))
+		return;
+
+	if (prefix->bitlen == 128 &&
+	    IN6_IS_ADDR_LOOPBACK(&prefix->add.sin6) &&
+	    (data[0] == NULL || !* (bool *) data[0]) &&
+	    (data[2] == NULL || !* (bool *) data[2]) &&
+	    (data[3] == NULL || !* (bool *) data[3]))
+		return;
 
 	/* Non-negated, non-loopback */
-	insecure_prefix_found = ISC_TRUE;	/* LOCKED */
+	insecure_prefix_found = true;	/* LOCKED */
 	return;
 }
 
 /*
- * Return ISC_TRUE iff the acl 'a' is considered insecure, that is,
+ * Return true iff the acl 'a' is considered insecure, that is,
  * if it contains IP addresses other than those of the local host.
  * This is intended for applications such as printing warning
  * messages for suspect ACLs; it is not intended for making access
  * control decisions.  We make no guarantee that an ACL for which
- * this function returns ISC_FALSE is safe.
+ * this function returns false is safe.
  */
-isc_boolean_t
+bool
 dns_acl_isinsecure(const dns_acl_t *a) {
 	unsigned int i;
-	isc_boolean_t insecure;
+	bool insecure;
 
 	RUNTIME_CHECK(isc_once_do(&insecure_prefix_once,
 				  initialize_action) == ISC_R_SUCCESS);
@@ -558,12 +640,12 @@ dns_acl_isinsecure(const dns_acl_t *a) {
 	 * non-loopback prefixes.
 	 */
 	LOCK(&insecure_prefix_lock);
-	insecure_prefix_found = ISC_FALSE;
+	insecure_prefix_found = false;
 	isc_radix_process(a->iptable->radix, is_insecure);
 	insecure = insecure_prefix_found;
 	UNLOCK(&insecure_prefix_lock);
 	if (insecure)
-		return(ISC_TRUE);
+		return (true);
 
 	/* Now check non-radix elements */
 	for (i = 0; i < a->length; i++) {
@@ -580,20 +662,44 @@ dns_acl_isinsecure(const dns_acl_t *a) {
 
 		case dns_aclelementtype_nestedacl:
 			if (dns_acl_isinsecure(e->nestedacl))
-				return (ISC_TRUE);
+				return (true);
 			continue;
 
+#ifdef HAVE_GEOIP
+		case dns_aclelementtype_geoip:
+#endif
 		case dns_aclelementtype_localnets:
-			return (ISC_TRUE);
+			return (true);
 
 		default:
 			INSIST(0);
-			return (ISC_TRUE);
+			ISC_UNREACHABLE();
 		}
 	}
 
 	/* No insecure elements were found. */
-	return (ISC_FALSE);
+	return (false);
+}
+
+
+/*%
+ * Check whether an address/signer is allowed by a given acl/aclenv.
+ */
+bool
+dns_acl_allowed(isc_netaddr_t *addr, dns_name_t *signer,
+		isc_netaddr_t *ecs_addr, uint8_t ecs_addrlen,
+		uint8_t *ecs_scope, dns_acl_t *acl, dns_aclenv_t *aclenv)
+{
+	int match;
+	isc_result_t result;
+
+	if (acl == NULL)
+		return (true);
+	result = dns_acl_match2(addr, signer, ecs_addr, ecs_addrlen,
+				ecs_scope, acl, aclenv, &match, NULL);
+	if (result == ISC_R_SUCCESS && match > 0)
+		return (true);
+	return (false);
 }
 
 /*
@@ -611,7 +717,11 @@ dns_aclenv_init(isc_mem_t *mctx, dns_aclenv_t *env) {
 	result = dns_acl_create(mctx, 0, &env->localnets);
 	if (result != ISC_R_SUCCESS)
 		goto cleanup_localhost;
-	env->match_mapped = ISC_FALSE;
+	env->match_mapped = false;
+#ifdef HAVE_GEOIP
+	env->geoip = NULL;
+	env->geoip_use_ecs = false;
+#endif
 	return (ISC_R_SUCCESS);
 
  cleanup_localhost:
@@ -627,10 +737,16 @@ dns_aclenv_copy(dns_aclenv_t *t, dns_aclenv_t *s) {
 	dns_acl_detach(&t->localnets);
 	dns_acl_attach(s->localnets, &t->localnets);
 	t->match_mapped = s->match_mapped;
+#ifdef HAVE_GEOIP
+	t->geoip = s->geoip;
+	t->geoip_use_ecs = s->geoip_use_ecs;
+#endif
 }
 
 void
 dns_aclenv_destroy(dns_aclenv_t *env) {
-	dns_acl_detach(&env->localhost);
-	dns_acl_detach(&env->localnets);
+	if (env->localhost != NULL)
+		dns_acl_detach(&env->localhost);
+	if (env->localnets != NULL)
+		dns_acl_detach(&env->localnets);
 }
