@@ -19,6 +19,7 @@
  ***/
 
 #include <stdio.h>
+#include <stdbool.h>
 
 #include <isc/formatcheck.h>
 #include <isc/lang.h>
@@ -178,9 +179,9 @@ dns_zone_getclass(dns_zone_t *zone);
  */
 
 isc_result_t
-dns_zone_getserial2(dns_zone_t *zone, isc_uint32_t *serialp);
+dns_zone_getserial2(dns_zone_t *zone, uint32_t *serialp);
 
-isc_uint32_t
+uint32_t
 dns_zone_getserial(dns_zone_t *zone);
 /*%<
  *	Returns the current serial number of the zone.  On success, the SOA
@@ -319,7 +320,7 @@ dns_zone_getfile(dns_zone_t *zone);
  */
 
 void
-dns_zone_setmaxrecords(dns_zone_t *zone, isc_uint32_t records);
+dns_zone_setmaxrecords(dns_zone_t *zone, uint32_t records);
 /*%<
  * 	Sets the maximim number of records permitted in a zone.
  *	0 implies unlimited.
@@ -331,7 +332,7 @@ dns_zone_setmaxrecords(dns_zone_t *zone, isc_uint32_t records);
  *\li	void
  */
 
-isc_uint32_t
+uint32_t
 dns_zone_getmaxrecords(dns_zone_t *zone);
 /*%<
  * 	Gets the maximim number of records permitted in a zone.
@@ -341,11 +342,11 @@ dns_zone_getmaxrecords(dns_zone_t *zone);
  *\li	'zone' to be valid initialised zone.
  *
  * Returns:
- *\li	isc_uint32_t maxrecords.
+ *\li	uint32_t maxrecords.
  */
 
 void
-dns_zone_setmaxttl(dns_zone_t *zone, isc_uint32_t maxttl);
+dns_zone_setmaxttl(dns_zone_t *zone, uint32_t maxttl);
 /*%<
  * 	Sets the max ttl of the zone.
  *
@@ -403,6 +404,10 @@ dns_zone_loadandthaw(dns_zone_t *zone);
 
 isc_result_t
 dns_zone_asyncload(dns_zone_t *zone, dns_zt_zoneloaded_t done, void *arg);
+
+isc_result_t
+dns_zone_asyncload2(dns_zone_t *zone, dns_zt_zoneloaded_t done, void *arg,
+		    bool newonly);
 /*%<
  * Cause the database to be loaded from its backing store asynchronously.
  * Other zone maintenance functions are suspended until this is complete.
@@ -421,7 +426,7 @@ dns_zone_asyncload(dns_zone_t *zone, dns_zt_zoneloaded_t done, void *arg);
  *\li	#ISC_R_NOMEMORY
  */
 
-isc_boolean_t
+bool
 dns__zone_loadpending(dns_zone_t *zone);
 /*%<
  * Indicates whether the zone is waiting to be loaded asynchronously.
@@ -478,9 +483,9 @@ dns_zone_idetach(dns_zone_t **zonep);
  */
 
 void
-dns_zone_setflag(dns_zone_t *zone, unsigned int flags, isc_boolean_t value);
+dns_zone_setflag(dns_zone_t *zone, unsigned int flags, bool value);
 /*%<
- *	Sets ('value' == 'ISC_TRUE') / clears ('value' == 'IS_FALSE')
+ *	Sets ('value' == 'true') / clears ('value' == 'IS_FALSE')
  *	zone flags.  Valid flag bits are DNS_ZONE_F_*.
  *
  * Requires
@@ -610,7 +615,7 @@ dns_zone_dumptostream2(dns_zone_t *zone, FILE *fd, dns_masterformat_t format,
 isc_result_t
 dns_zone_dumptostream3(dns_zone_t *zone, FILE *fd, dns_masterformat_t format,
 		       const dns_master_style_t *style,
-		       const isc_uint32_t rawversion);
+		       const uint32_t rawversion);
 /*%<
  *    Write the zone to stream 'fd' in the specified 'format'.
  *    If the 'format' is dns_masterformat_text (RFC1035), 'style' also
@@ -655,12 +660,12 @@ dns_zone_maintenance(dns_zone_t *zone);
 
 isc_result_t
 dns_zone_setmasters(dns_zone_t *zone, const isc_sockaddr_t *masters,
-		    isc_uint32_t count);
+		    uint32_t count);
 isc_result_t
 dns_zone_setmasterswithkeys(dns_zone_t *zone,
 			    const isc_sockaddr_t *masters,
 			    dns_name_t **keynames,
-			    isc_uint32_t count);
+			    uint32_t count);
 /*%<
  *	Set the list of master servers for the zone.
  *
@@ -683,14 +688,14 @@ dns_zone_setmasterswithkeys(dns_zone_t *zone,
 
 isc_result_t
 dns_zone_setalsonotify(dns_zone_t *zone, const isc_sockaddr_t *notify,
-		       isc_uint32_t count);
+		       uint32_t count);
 isc_result_t
 dns_zone_setalsonotifywithkeys(dns_zone_t *zone, const isc_sockaddr_t *notify,
-			       dns_name_t **keynames, isc_uint32_t count);
+			       dns_name_t **keynames, uint32_t count);
 isc_result_t
 dns_zone_setalsonotifydscpkeys(dns_zone_t *zone, const isc_sockaddr_t *notify,
 			       const isc_dscp_t *dscps, dns_name_t **keynames,
-			       isc_uint32_t count);
+			       uint32_t count);
 /*%<
  *	Set the list of additional servers to be notified when
  *	a zone changes.	 To clear the list use 'count = 0'.
@@ -719,13 +724,13 @@ dns_zone_unload(dns_zone_t *zone);
 
 void
 dns_zone_setoption(dns_zone_t *zone, unsigned int option,
-		   isc_boolean_t value);
+		   bool value);
 void
 dns_zone_setoption2(dns_zone_t *zone, unsigned int option,
-		    isc_boolean_t value);
+		    bool value);
 /*%<
- *	Set the given options on ('value' == ISC_TRUE) or off
- *	('value' == #ISC_FALSE).
+ *	Set the given options on ('value' == true) or off
+ *	('value' == #false).
  *
  *	dns_zone_setoption2() has been introduced because the number
  *	of options needed now exceeds the 32 bits in the zone->options
@@ -754,10 +759,10 @@ dns_zone_getoptions2(dns_zone_t *zone);
  */
 
 void
-dns_zone_setkeyopt(dns_zone_t *zone, unsigned int option, isc_boolean_t value);
+dns_zone_setkeyopt(dns_zone_t *zone, unsigned int option, bool value);
 /*%<
- *	Set key options on ('value' == ISC_TRUE) or off ('value' ==
- *	#ISC_FALSE).
+ *	Set key options on ('value' == true) or off ('value' ==
+ *	#false).
  *
  * Require:
  *\li	'zone' to be a valid zone.
@@ -773,7 +778,7 @@ dns_zone_getkeyopts(dns_zone_t *zone);
  */
 
 void
-dns_zone_setminrefreshtime(dns_zone_t *zone, isc_uint32_t val);
+dns_zone_setminrefreshtime(dns_zone_t *zone, uint32_t val);
 /*%<
  *	Set the minimum refresh time.
  *
@@ -783,7 +788,7 @@ dns_zone_setminrefreshtime(dns_zone_t *zone, isc_uint32_t val);
  */
 
 void
-dns_zone_setmaxrefreshtime(dns_zone_t *zone, isc_uint32_t val);
+dns_zone_setmaxrefreshtime(dns_zone_t *zone, uint32_t val);
 /*%<
  *	Set the maximum refresh time.
  *
@@ -793,7 +798,7 @@ dns_zone_setmaxrefreshtime(dns_zone_t *zone, isc_uint32_t val);
  */
 
 void
-dns_zone_setminretrytime(dns_zone_t *zone, isc_uint32_t val);
+dns_zone_setminretrytime(dns_zone_t *zone, uint32_t val);
 /*%<
  *	Set the minimum retry time.
  *
@@ -803,7 +808,7 @@ dns_zone_setminretrytime(dns_zone_t *zone, isc_uint32_t val);
  */
 
 void
-dns_zone_setmaxretrytime(dns_zone_t *zone, isc_uint32_t val);
+dns_zone_setmaxretrytime(dns_zone_t *zone, uint32_t val);
 /*%<
  *	Set the maximum retry time.
  *
@@ -1199,7 +1204,7 @@ dns_zone_clearxfracl(dns_zone_t *zone);
  *\li	'zone' to be a valid zone.
  */
 
-isc_boolean_t
+bool
 dns_zone_getupdatedisabled(dns_zone_t *zone);
 /*%<
  * Return update disabled.
@@ -1207,7 +1212,7 @@ dns_zone_getupdatedisabled(dns_zone_t *zone);
  */
 
 void
-dns_zone_setupdatedisabled(dns_zone_t *zone, isc_boolean_t state);
+dns_zone_setupdatedisabled(dns_zone_t *zone, bool state);
 /*%<
  * Set update disabled.
  * Should only be called only when running in isc_task_exclusive() mode.
@@ -1215,14 +1220,14 @@ dns_zone_setupdatedisabled(dns_zone_t *zone, isc_boolean_t state);
  * call has been made.
  */
 
-isc_boolean_t
+bool
 dns_zone_getzeronosoattl(dns_zone_t *zone);
 /*%<
  * Return zero-no-soa-ttl status.
  */
 
 void
-dns_zone_setzeronosoattl(dns_zone_t *zone, isc_boolean_t state);
+dns_zone_setzeronosoattl(dns_zone_t *zone, bool state);
 /*%<
  * Set zero-no-soa-ttl status.
  */
@@ -1246,7 +1251,7 @@ dns_zone_getchecknames(dns_zone_t *zone);
  */
 
 void
-dns_zone_setjournalsize(dns_zone_t *zone, isc_int32_t size);
+dns_zone_setjournalsize(dns_zone_t *zone, int32_t size);
 /*%<
  *	Sets the journal size for the zone.
  *
@@ -1254,7 +1259,7 @@ dns_zone_setjournalsize(dns_zone_t *zone, isc_int32_t size);
  *\li	'zone' to be a valid zone.
  */
 
-isc_int32_t
+int32_t
 dns_zone_getjournalsize(dns_zone_t *zone);
 /*%<
  *	Return the journal size as set with a previous call to
@@ -1288,7 +1293,7 @@ dns_zone_notifyreceive2(dns_zone_t *zone, isc_sockaddr_t *from,
  */
 
 void
-dns_zone_setmaxxfrin(dns_zone_t *zone, isc_uint32_t maxxfrin);
+dns_zone_setmaxxfrin(dns_zone_t *zone, uint32_t maxxfrin);
 /*%<
  * Set the maximum time (in seconds) that a zone transfer in (AXFR/IXFR)
  * of this zone will use before being aborted.
@@ -1297,7 +1302,7 @@ dns_zone_setmaxxfrin(dns_zone_t *zone, isc_uint32_t maxxfrin);
  * \li	'zone' to be valid initialised zone.
  */
 
-isc_uint32_t
+uint32_t
 dns_zone_getmaxxfrin(dns_zone_t *zone);
 /*%<
  * Returns the maximum transfer time for this zone.  This will be
@@ -1309,7 +1314,7 @@ dns_zone_getmaxxfrin(dns_zone_t *zone);
  */
 
 void
-dns_zone_setmaxxfrout(dns_zone_t *zone, isc_uint32_t maxxfrout);
+dns_zone_setmaxxfrout(dns_zone_t *zone, uint32_t maxxfrout);
 /*%<
  * Set the maximum time (in seconds) that a zone transfer out (AXFR/IXFR)
  * of this zone will use before being aborted.
@@ -1318,7 +1323,7 @@ dns_zone_setmaxxfrout(dns_zone_t *zone, isc_uint32_t maxxfrout);
  * \li	'zone' to be valid initialised zone.
  */
 
-isc_uint32_t
+uint32_t
 dns_zone_getmaxxfrout(dns_zone_t *zone);
 /*%<
  * Returns the maximum transfer time for this zone.  This will be
@@ -1410,15 +1415,15 @@ dns_zone_notify(dns_zone_t *zone);
  */
 
 isc_result_t
-dns_zone_replacedb(dns_zone_t *zone, dns_db_t *db, isc_boolean_t dump);
+dns_zone_replacedb(dns_zone_t *zone, dns_db_t *db, bool dump);
 /*%<
  * Replace the database of "zone" with a new database "db".
  *
- * If "dump" is ISC_TRUE, then the new zone contents are dumped
+ * If "dump" is true, then the new zone contents are dumped
  * into to the zone's master file for persistence.  When replacing
  * a zone database by one just loaded from a master file, set
- * "dump" to ISC_FALSE to avoid a redundant redump of the data just
- * loaded.  Otherwise, it should be set to ISC_TRUE.
+ * "dump" to false to avoid a redundant redump of the data just
+ * loaded.  Otherwise, it should be set to true.
  *
  * If the "diff-on-reload" option is enabled in the configuration file,
  * the differences between the old and the new database are added to the
@@ -1435,7 +1440,7 @@ dns_zone_replacedb(dns_zone_t *zone, dns_db_t *db, isc_boolean_t dump);
  *	Others
  */
 
-isc_uint32_t
+uint32_t
 dns_zone_getidlein(dns_zone_t *zone);
 /*%<
  * Requires:
@@ -1446,7 +1451,7 @@ dns_zone_getidlein(dns_zone_t *zone);
  */
 
 void
-dns_zone_setidlein(dns_zone_t *zone, isc_uint32_t idlein);
+dns_zone_setidlein(dns_zone_t *zone, uint32_t idlein);
 /*%<
  * \li	Set the idle timeout for transfer the.
  * \li	Zero set the default value, 1 hour.
@@ -1455,7 +1460,7 @@ dns_zone_setidlein(dns_zone_t *zone, isc_uint32_t idlein);
  * \li	'zone' to be a valid zone.
  */
 
-isc_uint32_t
+uint32_t
 dns_zone_getidleout(dns_zone_t *zone);
 /*%<
  *
@@ -1467,7 +1472,7 @@ dns_zone_getidleout(dns_zone_t *zone);
  */
 
 void
-dns_zone_setidleout(dns_zone_t *zone, isc_uint32_t idleout);
+dns_zone_setidleout(dns_zone_t *zone, uint32_t idleout);
 /*%<
  * \li	Set the idle timeout for transfers out.
  * \li	Zero set the default value, 1 hour.
@@ -1513,7 +1518,7 @@ dns_zone_getmgr(dns_zone_t *zone);
  */
 
 void
-dns_zone_setsigvalidityinterval(dns_zone_t *zone, isc_uint32_t interval);
+dns_zone_setsigvalidityinterval(dns_zone_t *zone, uint32_t interval);
 /*%<
  * Set the zone's RRSIG validity interval.  This is the length of time
  * for which DNSSEC signatures created as a result of dynamic updates
@@ -1523,7 +1528,7 @@ dns_zone_setsigvalidityinterval(dns_zone_t *zone, isc_uint32_t interval);
  * \li	'zone' to be a valid zone.
  */
 
-isc_uint32_t
+uint32_t
 dns_zone_getsigvalidityinterval(dns_zone_t *zone);
 /*%<
  * Get the zone's RRSIG validity interval.
@@ -1533,7 +1538,7 @@ dns_zone_getsigvalidityinterval(dns_zone_t *zone);
  */
 
 void
-dns_zone_setsigresigninginterval(dns_zone_t *zone, isc_uint32_t interval);
+dns_zone_setsigresigninginterval(dns_zone_t *zone, uint32_t interval);
 /*%<
  * Set the zone's RRSIG re-signing interval.  A dynamic zone's RRSIG's
  * will be re-signed 'interval' amount of time before they expire.
@@ -1542,7 +1547,7 @@ dns_zone_setsigresigninginterval(dns_zone_t *zone, isc_uint32_t interval);
  * \li	'zone' to be a valid zone.
  */
 
-isc_uint32_t
+uint32_t
 dns_zone_getsigresigninginterval(dns_zone_t *zone);
 /*%<
  * Get the zone's RRSIG re-signing interval.
@@ -1759,7 +1764,7 @@ dns_zonemgr_releasezone(dns_zonemgr_t *zmgr, dns_zone_t *zone);
  */
 
 void
-dns_zonemgr_settransfersin(dns_zonemgr_t *zmgr, isc_uint32_t value);
+dns_zonemgr_settransfersin(dns_zonemgr_t *zmgr, uint32_t value);
 /*%<
  *	Set the maximum number of simultaneous transfers in allowed by
  *	the zone manager.
@@ -1768,7 +1773,7 @@ dns_zonemgr_settransfersin(dns_zonemgr_t *zmgr, isc_uint32_t value);
  *\li	'zmgr' to be a valid zone manager.
  */
 
-isc_uint32_t
+uint32_t
 dns_zonemgr_getttransfersin(dns_zonemgr_t *zmgr);
 /*%<
  *	Return the maximum number of simultaneous transfers in allowed.
@@ -1778,7 +1783,7 @@ dns_zonemgr_getttransfersin(dns_zonemgr_t *zmgr);
  */
 
 void
-dns_zonemgr_settransfersperns(dns_zonemgr_t *zmgr, isc_uint32_t value);
+dns_zonemgr_settransfersperns(dns_zonemgr_t *zmgr, uint32_t value);
 /*%<
  *	Set the number of zone transfers allowed per nameserver.
  *
@@ -1786,7 +1791,7 @@ dns_zonemgr_settransfersperns(dns_zonemgr_t *zmgr, isc_uint32_t value);
  *\li	'zmgr' to be a valid zone manager
  */
 
-isc_uint32_t
+uint32_t
 dns_zonemgr_getttransfersperns(dns_zonemgr_t *zmgr);
 /*%<
  *	Return the number of transfers allowed per nameserver.
@@ -1796,7 +1801,7 @@ dns_zonemgr_getttransfersperns(dns_zonemgr_t *zmgr);
  */
 
 void
-dns_zonemgr_setiolimit(dns_zonemgr_t *zmgr, isc_uint32_t iolimit);
+dns_zonemgr_setiolimit(dns_zonemgr_t *zmgr, uint32_t iolimit);
 /*%<
  *	Set the number of simultaneous file descriptors available for
  *	reading and writing masterfiles.
@@ -1806,7 +1811,7 @@ dns_zonemgr_setiolimit(dns_zonemgr_t *zmgr, isc_uint32_t iolimit);
  *\li	'iolimit' to be positive.
  */
 
-isc_uint32_t
+uint32_t
 dns_zonemgr_getiolimit(dns_zonemgr_t *zmgr);
 /*%<
  *	Get the number of simultaneous file descriptors available for
@@ -1892,11 +1897,11 @@ dns_zonemgr_unreachableadd(dns_zonemgr_t *zmgr, isc_sockaddr_t *remote,
  *\li	'local' to be a valid sockaddr.
  */
 
-isc_boolean_t
+bool
 dns_zonemgr_unreachable(dns_zonemgr_t *zmgr, isc_sockaddr_t *remote,
 			isc_sockaddr_t *local, isc_time_t *now);
 /*%<
- *	Returns ISC_TRUE if the given local/remote address pair
+ *	Returns true if the given local/remote address pair
  *	is found in the zone maanger's unreachable cache.
  *
  * Requires:
@@ -1927,7 +1932,7 @@ dns_zone_forcereload(dns_zone_t *zone);
  *\li      'zone' to be a valid zone.
  */
 
-isc_boolean_t
+bool
 dns_zone_isforced(dns_zone_t *zone);
 /*%<
  *      Check if the zone is waiting a forced reload.
@@ -1937,12 +1942,12 @@ dns_zone_isforced(dns_zone_t *zone);
  */
 
 isc_result_t
-dns_zone_setstatistics(dns_zone_t *zone, isc_boolean_t on);
+dns_zone_setstatistics(dns_zone_t *zone, bool on);
 /*%<
  * This function is obsoleted by dns_zone_setrequeststats().
  */
 
-isc_uint64_t *
+uint64_t *
 dns_zone_getstatscounters(dns_zone_t *zone);
 /*%<
  * This function is obsoleted by dns_zone_getrequeststats().
@@ -2106,7 +2111,7 @@ dns_zone_setcheckns(dns_zone_t *zone, dns_checknsfunc_t checkns);
  */
 
 void
-dns_zone_setnotifydelay(dns_zone_t *zone, isc_uint32_t delay);
+dns_zone_setnotifydelay(dns_zone_t *zone, uint32_t delay);
 /*%<
  * Set the minimum delay between sets of notify messages.
  *
@@ -2114,7 +2119,7 @@ dns_zone_setnotifydelay(dns_zone_t *zone, isc_uint32_t delay);
  *	'zone' to be valid.
  */
 
-isc_uint32_t
+uint32_t
 dns_zone_getnotifydelay(dns_zone_t *zone);
 /*%<
  * Get the minimum delay between sets of notify messages.
@@ -2128,29 +2133,29 @@ dns_zone_setisself(dns_zone_t *zone, dns_isselffunc_t isself, void *arg);
 /*%<
  * Set the isself callback function and argument.
  *
- * isc_boolean_t
+ * bool
  * isself(dns_view_t *myview, dns_tsigkey_t *mykey,
  *	  const isc_netaddr_t *srcaddr, const isc_netaddr_t *destaddr,
  *	  dns_rdataclass_t rdclass, void *arg);
  *
- * 'isself' returns ISC_TRUE if a non-recursive query from 'srcaddr' to
+ * 'isself' returns true if a non-recursive query from 'srcaddr' to
  * 'destaddr' with optional key 'mykey' for class 'rdclass' would be
  * delivered to 'myview'.
  */
 
 void
-dns_zone_setnodes(dns_zone_t *zone, isc_uint32_t nodes);
+dns_zone_setnodes(dns_zone_t *zone, uint32_t nodes);
 /*%<
  * Set the number of nodes that will be checked per quantum.
  */
 
 void
-dns_zone_setsignatures(dns_zone_t *zone, isc_uint32_t signatures);
+dns_zone_setsignatures(dns_zone_t *zone, uint32_t signatures);
 /*%<
  * Set the number of signatures that will be generated per quantum.
  */
 
-isc_uint32_t
+uint32_t
 dns_zone_getsignatures(dns_zone_t *zone);
 /*%<
  * Get the number of signatures that will be generated per quantum.
@@ -2158,7 +2163,7 @@ dns_zone_getsignatures(dns_zone_t *zone);
 
 isc_result_t
 dns_zone_signwithkey(dns_zone_t *zone, dns_secalg_t algorithm,
-		     isc_uint16_t keyid, isc_boolean_t deleteit);
+		     uint16_t keyid, bool deleteit);
 /*%<
  * Initiate/resume signing of the entire zone with the zone DNSKEY(s)
  * that match the given algorithm and keyid.
@@ -2180,7 +2185,7 @@ dns_zone_getprivatetype(dns_zone_t *zone);
  */
 
 void
-dns_zone_rekey(dns_zone_t *zone, isc_boolean_t fullsign);
+dns_zone_rekey(dns_zone_t *zone, bool fullsign);
 /*%<
  * Update the zone's DNSKEY set from the key repository.
  *
@@ -2226,19 +2231,19 @@ dns_zone_cdscheck(dns_zone_t *zone, dns_db_t *db, dns_dbversion_t *version);
  */
 
 void
-dns_zone_setadded(dns_zone_t *zone, isc_boolean_t added);
+dns_zone_setadded(dns_zone_t *zone, bool added);
 /*%
- * Sets the value of zone->added, which should be ISC_TRUE for
+ * Sets the value of zone->added, which should be true for
  * zones that were originally added by "rndc addzone".
  *
  * Requires:
  * \li	'zone' to be valid.
  */
 
-isc_boolean_t
+bool
 dns_zone_getadded(dns_zone_t *zone);
 /*%
- * Returns ISC_TRUE if the zone was originally added at runtime
+ * Returns true if the zone was originally added at runtime
  * using "rndc addzone".
  *
  * Requires:
@@ -2246,19 +2251,19 @@ dns_zone_getadded(dns_zone_t *zone);
  */
 
 void
-dns_zone_setautomatic(dns_zone_t *zone, isc_boolean_t automatic);
+dns_zone_setautomatic(dns_zone_t *zone, bool automatic);
 /*%
- * Sets the value of zone->automatic, which should be ISC_TRUE for
+ * Sets the value of zone->automatic, which should be true for
  * zones that were automatically added by named.
  *
  * Requires:
  * \li	'zone' to be valid.
  */
 
-isc_boolean_t
+bool
 dns_zone_getautomatic(dns_zone_t *zone);
 /*%
- * Returns ISC_TRUE if the zone was added automatically by named.
+ * Returns true if the zone was added automatically by named.
  *
  * Requires:
  * \li	'zone' to be valid.
@@ -2270,8 +2275,8 @@ dns_zone_dlzpostload(dns_zone_t *zone, dns_db_t *db);
  * Load the origin names for a writeable DLZ database.
  */
 
-isc_boolean_t
-dns_zone_isdynamic(dns_zone_t *zone, isc_boolean_t ignore_freeze);
+bool
+dns_zone_isdynamic(dns_zone_t *zone, bool ignore_freeze);
 /*%
  * Return true iff the zone is "dynamic", in the sense that the zone's
  * master file (if any) is written by the server, rather than being
@@ -2289,7 +2294,7 @@ dns_zone_isdynamic(dns_zone_t *zone, isc_boolean_t ignore_freeze);
  */
 
 isc_result_t
-dns_zone_setrefreshkeyinterval(dns_zone_t *zone, isc_uint32_t interval);
+dns_zone_setrefreshkeyinterval(dns_zone_t *zone, uint32_t interval);
 /*%
  * Sets the frequency, in minutes, with which the key repository will be
  * checked to see if the keys for this zone have been updated.  Any value
@@ -2300,7 +2305,7 @@ dns_zone_setrefreshkeyinterval(dns_zone_t *zone, isc_uint32_t interval);
  * \li	'zone' to be valid.
  */
 
-isc_boolean_t
+bool
 dns_zone_getrequestexpire(dns_zone_t *zone);
 /*%
  * Returns the true/false value of the request-expire option in the zone.
@@ -2310,7 +2315,7 @@ dns_zone_getrequestexpire(dns_zone_t *zone);
  */
 
 void
-dns_zone_setrequestexpire(dns_zone_t *zone, isc_boolean_t flag);
+dns_zone_setrequestexpire(dns_zone_t *zone, bool flag);
 /*%
  * Sets the request-expire option for the zone. Either true or false. The
  * default value is determined by the setting of this option in the view.
@@ -2320,7 +2325,7 @@ dns_zone_setrequestexpire(dns_zone_t *zone, isc_boolean_t flag);
  */
 
 
-isc_boolean_t
+bool
 dns_zone_getrequestixfr(dns_zone_t *zone);
 /*%
  * Returns the true/false value of the request-ixfr option in the zone.
@@ -2330,7 +2335,7 @@ dns_zone_getrequestixfr(dns_zone_t *zone);
  */
 
 void
-dns_zone_setrequestixfr(dns_zone_t *zone, isc_boolean_t flag);
+dns_zone_setrequestixfr(dns_zone_t *zone, bool flag);
 /*%
  * Sets the request-ixfr option for the zone. Either true or false. The
  * default value is determined by the setting of this option in the view.
@@ -2370,13 +2375,13 @@ isc_result_t
 dns_zone_keydone(dns_zone_t *zone, const char *data);
 
 isc_result_t
-dns_zone_setnsec3param(dns_zone_t *zone, isc_uint8_t hash, isc_uint8_t flags,
-		       isc_uint16_t iter, isc_uint8_t saltlen,
-		       unsigned char *salt, isc_boolean_t replace);
+dns_zone_setnsec3param(dns_zone_t *zone, uint8_t hash, uint8_t flags,
+		       uint16_t iter, uint8_t saltlen,
+		       unsigned char *salt, bool replace);
 /*%
  * Set the NSEC3 parameters for the zone.
  *
- * If 'replace' is ISC_TRUE, then the existing NSEC3 chain, if any, will
+ * If 'replace' is true, then the existing NSEC3 chain, if any, will
  * be replaced with the new one.  If 'hash' is zero, then the replacement
  * chain will be NSEC rather than NSEC3.
  *
@@ -2504,7 +2509,7 @@ dns_zone_getstatlevel(dns_zone_t *zone);
  */
 
 isc_result_t
-dns_zone_setserial(dns_zone_t *zone, isc_uint32_t serial);
+dns_zone_setserial(dns_zone_t *zone, uint32_t serial);
 /*%
  * Set the zone's serial to 'serial'.
  */
