@@ -1,26 +1,22 @@
 /*
- * Copyright (C) 2004-2008, 2011-2013  Internet Systems Consortium, Inc. ("ISC")
- * Copyright (C) 1999-2003  Internet Software Consortium.
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
-/* $Id$ */
 
 /*! \file */
 
 #include <config.h>
 
+#include <inttypes.h>
+#include <stdbool.h>
+
 #include <isc/mem.h>
 #include <isc/print.h>
 #include <isc/random.h>
@@ -30,6 +26,7 @@
 #include <isc/util.h>
 
 #include <dns/callbacks.h>
+#include <dns/catz.h>
 #include <dns/db.h>
 #include <dns/diff.h>
 #include <dns/events.h>
@@ -107,12 +104,13 @@ struct dns_xfrin_ctx {
 	int			connects; 	/*%< Connect in progress */
 	int			sends;		/*%< Send in progress */
 	int			recvs;	  	/*%< Receive in progress */
-	isc_boolean_t		shuttingdown;
+	bool		shuttingdown;
+	isc_result_t		shutdown_result;
 
 	dns_name_t 		name; 		/*%< Name of zone to transfer */
 	dns_rdataclass_t 	rdclass;
 
-	isc_boolean_t		checkid;
+	bool		checkid;
 	dns_messageid_t		id;
 
 	/*%
@@ -133,7 +131,7 @@ struct dns_xfrin_ctx {
 
 	/*% Incoming reply TCP message */
 	dns_tcpmsg_t		tcpmsg;
-	isc_boolean_t		tcpmsg_valid;
+	bool		tcpmsg_valid;
 
 	dns_db_t 		*db;
 	dns_dbversion_t 	*ver;
@@ -141,12 +139,15 @@ struct dns_xfrin_ctx {
 	int 			difflen;	/*%< Number of pending tuples */
 
 	xfrin_state_t 		state;
-	isc_uint32_t 		end_serial;
-	isc_boolean_t 		is_ixfr;
+	uint32_t 		end_serial;
+	bool 		is_ixfr;
 
 	unsigned int		nmsg;		/*%< Number of messages recvd */
 	unsigned int		nrecs;		/*%< Number of records recvd */
-	isc_uint64_t		nbytes;		/*%< Number of bytes received */
+	uint64_t		nbytes;		/*%< Number of bytes received */
+
+	unsigned int		maxrecords;	/*%< The maximum number of
+						     records set for the zone */
 
 	isc_time_t		start;		/*%< Start time of the transfer */
 	isc_time_t		end;		/*%< End time of the transfer */
@@ -166,8 +167,8 @@ struct dns_xfrin_ctx {
 	dns_rdatacallbacks_t	axfr;
 
 	struct {
-		isc_uint32_t 	request_serial;
-		isc_uint32_t 	current_serial;
+		uint32_t 	request_serial;
+		uint32_t 	current_serial;
 		dns_journal_t	*journal;
 
 	} ixfr;
@@ -191,8 +192,8 @@ xfrin_create(isc_mem_t *mctx,
 	     dns_name_t *zonename,
 	     dns_rdataclass_t rdclass,
 	     dns_rdatatype_t reqtype,
-	     isc_sockaddr_t *masteraddr,
-	     isc_sockaddr_t *sourceaddr,
+	     const isc_sockaddr_t *masteraddr,
+	     const isc_sockaddr_t *sourceaddr,
 	     isc_dscp_t dscp,
 	     dns_tsigkey_t *tsigkey,
 	     dns_xfrin_ctx_t **xfrp);
@@ -214,7 +215,7 @@ static isc_result_t ixfr_putdata(dns_xfrin_ctx_t *xfr, dns_diffop_t op,
 static isc_result_t ixfr_commit(dns_xfrin_ctx_t *xfr);
 
 static isc_result_t xfr_rr(dns_xfrin_ctx_t *xfr, dns_name_t *name,
-			   isc_uint32_t ttl, dns_rdata_t *rdata);
+			   uint32_t ttl, dns_rdata_t *rdata);
 
 static isc_result_t xfrin_start(dns_xfrin_ctx_t *xfr);
 
@@ -232,12 +233,12 @@ static isc_result_t
 render(dns_message_t *msg, isc_mem_t *mctx, isc_buffer_t *buf);
 
 static void
-xfrin_logv(int level, const char *zonetext, isc_sockaddr_t *masteraddr,
+xfrin_logv(int level, const char *zonetext, const isc_sockaddr_t *masteraddr,
 	   const char *fmt, va_list ap)
      ISC_FORMAT_PRINTF(4, 0);
 
 static void
-xfrin_log1(int level, const char *zonetext, isc_sockaddr_t *masteraddr,
+xfrin_log1(int level, const char *zonetext, const isc_sockaddr_t *masteraddr,
 	   const char *fmt, ...)
      ISC_FORMAT_PRINTF(4, 5);
 
@@ -254,7 +255,7 @@ static isc_result_t
 axfr_init(dns_xfrin_ctx_t *xfr) {
 	isc_result_t result;
 
-	xfr->is_ixfr = ISC_FALSE;
+	xfr->is_ixfr = false;
 
 	if (xfr->db != NULL)
 		dns_db_detach(&xfr->db);
@@ -278,8 +279,10 @@ axfr_makedb(dns_xfrin_ctx_t *xfr, dns_db_t **dbp) {
 			       xfr->rdclass,
 			       0, NULL, /* XXX guess */
 			       dbp);
-	if (result == ISC_R_SUCCESS)
+	if (result == ISC_R_SUCCESS) {
 		dns_zone_rpz_enable_db(xfr->zone, *dbp);
+		dns_zone_catz_enable_db(xfr->zone, *dbp);
+	}
 	return (result);
 }
 
@@ -291,6 +294,9 @@ axfr_putdata(dns_xfrin_ctx_t *xfr, dns_diffop_t op,
 
 	dns_difftuple_t *tuple = NULL;
 
+	if (rdata->rdclass != xfr->rdclass)
+		return(DNS_R_BADCLASS);
+
 	CHECK(dns_zone_checknames(xfr->zone, name, rdata));
 	CHECK(dns_difftuple_create(xfr->diff.mctx, op,
 				   name, ttl, rdata, &tuple));
@@ -308,10 +314,18 @@ axfr_putdata(dns_xfrin_ctx_t *xfr, dns_diffop_t op,
 static isc_result_t
 axfr_apply(dns_xfrin_ctx_t *xfr) {
 	isc_result_t result;
+	uint64_t records;
 
 	CHECK(dns_diff_load(&xfr->diff, xfr->axfr.add, xfr->axfr.add_private));
 	xfr->difflen = 0;
 	dns_diff_clear(&xfr->diff);
+	if (xfr->maxrecords != 0U) {
+		result = dns_db_getsize(xfr->db, xfr->ver, &records, NULL);
+		if (result == ISC_R_SUCCESS && records > xfr->maxrecords) {
+			result = DNS_R_TOOMANYRECORDS;
+			goto failure;
+		}
+	}
 	result = ISC_R_SUCCESS;
  failure:
 	return (result);
@@ -333,7 +347,7 @@ static isc_result_t
 axfr_finalize(dns_xfrin_ctx_t *xfr) {
 	isc_result_t result;
 
-	CHECK(dns_zone_replacedb(xfr->zone, xfr->db, ISC_TRUE));
+	CHECK(dns_zone_replacedb(xfr->zone, xfr->db, true));
 
 	result = ISC_R_SUCCESS;
  failure:
@@ -356,7 +370,7 @@ ixfr_init(dns_xfrin_ctx_t *xfr) {
 		return (DNS_R_FORMERR);
 	}
 
-	xfr->is_ixfr = ISC_TRUE;
+	xfr->is_ixfr = true;
 	INSIST(xfr->db != NULL);
 	xfr->difflen = 0;
 
@@ -375,8 +389,11 @@ ixfr_putdata(dns_xfrin_ctx_t *xfr, dns_diffop_t op,
 	     dns_name_t *name, dns_ttl_t ttl, dns_rdata_t *rdata)
 {
 	isc_result_t result;
-
 	dns_difftuple_t *tuple = NULL;
+
+	if (rdata->rdclass != xfr->rdclass)
+		return(DNS_R_BADCLASS);
+
 	if (op == DNS_DIFFOP_ADD)
 		CHECK(dns_zone_checknames(xfr->zone, name, rdata));
 	CHECK(dns_difftuple_create(xfr->diff.mctx, op,
@@ -395,6 +412,7 @@ ixfr_putdata(dns_xfrin_ctx_t *xfr, dns_diffop_t op,
 static isc_result_t
 ixfr_apply(dns_xfrin_ctx_t *xfr) {
 	isc_result_t result;
+	uint64_t records;
 
 	if (xfr->ver == NULL) {
 		CHECK(dns_db_newversion(xfr->db, &xfr->ver));
@@ -402,6 +420,13 @@ ixfr_apply(dns_xfrin_ctx_t *xfr) {
 			CHECK(dns_journal_begin_transaction(xfr->ixfr.journal));
 	}
 	CHECK(dns_diff_apply(&xfr->diff, xfr->db, xfr->ver));
+	if (xfr->maxrecords != 0U) {
+		result = dns_db_getsize(xfr->db, xfr->ver, &records, NULL);
+		if (result == ISC_R_SUCCESS && records > xfr->maxrecords) {
+			result = DNS_R_TOOMANYRECORDS;
+			goto failure;
+		}
+	}
 	if (xfr->ixfr.journal != NULL) {
 		result = dns_journal_writediff(xfr->ixfr.journal, &xfr->diff);
 		if (result != ISC_R_SUCCESS)
@@ -423,7 +448,7 @@ ixfr_commit(dns_xfrin_ctx_t *xfr) {
 		/* XXX enter ready-to-commit state here */
 		if (xfr->ixfr.journal != NULL)
 			CHECK(dns_journal_commit(xfr->ixfr.journal));
-		dns_db_closeversion(xfr->db, &xfr->ver, ISC_TRUE);
+		dns_db_closeversion(xfr->db, &xfr->ver, true);
 		dns_zone_markdirty(xfr->zone);
 	}
 	result = ISC_R_SUCCESS;
@@ -441,7 +466,7 @@ ixfr_commit(dns_xfrin_ctx_t *xfr) {
  * state.
  */
 static isc_result_t
-xfr_rr(dns_xfrin_ctx_t *xfr, dns_name_t *name, isc_uint32_t ttl,
+xfr_rr(dns_xfrin_ctx_t *xfr, dns_name_t *name, uint32_t ttl,
        dns_rdata_t *rdata)
 {
 	isc_result_t result;
@@ -505,7 +530,7 @@ xfr_rr(dns_xfrin_ctx_t *xfr, dns_name_t *name, isc_uint32_t ttl,
 			FAIL(DNS_R_UPTODATE);
 		}
 		if (xfr->reqtype == dns_rdatatype_axfr)
-			xfr->checkid = ISC_FALSE;
+			xfr->checkid = false;
 		xfr->state = XFRST_FIRSTDATA;
 		break;
 
@@ -537,7 +562,7 @@ xfr_rr(dns_xfrin_ctx_t *xfr, dns_name_t *name, isc_uint32_t ttl,
 
 	case XFRST_IXFR_DEL:
 		if (rdata->type == dns_rdatatype_soa) {
-			isc_uint32_t soa_serial = dns_soa_getserial(rdata);
+			uint32_t soa_serial = dns_soa_getserial(rdata);
 			xfr->state = XFRST_IXFR_ADDSOA;
 			xfr->ixfr.current_serial = soa_serial;
 			goto redo;
@@ -553,7 +578,7 @@ xfr_rr(dns_xfrin_ctx_t *xfr, dns_name_t *name, isc_uint32_t ttl,
 
 	case XFRST_IXFR_ADD:
 		if (rdata->type == dns_rdatatype_soa) {
-			isc_uint32_t soa_serial = dns_soa_getserial(rdata);
+			uint32_t soa_serial = dns_soa_getserial(rdata);
 			if (soa_serial == xfr->end_serial) {
 				CHECK(ixfr_commit(xfr));
 				xfr->state = XFRST_IXFR_END;
@@ -595,9 +620,10 @@ xfr_rr(dns_xfrin_ctx_t *xfr, dns_name_t *name, isc_uint32_t ttl,
 	case XFRST_IXFR_END:
 		FAIL(DNS_R_EXTRADATA);
 		/* NOTREACHED */
+		/* FALLTHROUGH */
 	default:
 		INSIST(0);
-		break;
+		ISC_UNREACHABLE();
 	}
 	result = ISC_R_SUCCESS;
  failure:
@@ -606,7 +632,7 @@ xfr_rr(dns_xfrin_ctx_t *xfr, dns_name_t *name, isc_uint32_t ttl,
 
 isc_result_t
 dns_xfrin_create(dns_zone_t *zone, dns_rdatatype_t xfrtype,
-		 isc_sockaddr_t *masteraddr, dns_tsigkey_t *tsigkey,
+		 const isc_sockaddr_t *masteraddr, dns_tsigkey_t *tsigkey,
 		 isc_mem_t *mctx, isc_timermgr_t *timermgr,
 		 isc_socketmgr_t *socketmgr, isc_task_t *task,
 		 dns_xfrindone_t done, dns_xfrin_ctx_t **xfrp)
@@ -625,6 +651,7 @@ dns_xfrin_create(dns_zone_t *zone, dns_rdatatype_t xfrtype,
 		break;
 	default:
 		INSIST(0);
+		ISC_UNREACHABLE();
 	}
 
 	return(dns_xfrin_create3(zone, xfrtype, masteraddr, &sourceaddr,
@@ -634,7 +661,8 @@ dns_xfrin_create(dns_zone_t *zone, dns_rdatatype_t xfrtype,
 
 isc_result_t
 dns_xfrin_create2(dns_zone_t *zone, dns_rdatatype_t xfrtype,
-		  isc_sockaddr_t *masteraddr, isc_sockaddr_t *sourceaddr,
+		  const isc_sockaddr_t *masteraddr,
+		  const isc_sockaddr_t *sourceaddr,
 		  dns_tsigkey_t *tsigkey, isc_mem_t *mctx,
 		  isc_timermgr_t *timermgr, isc_socketmgr_t *socketmgr,
 		  isc_task_t *task, dns_xfrindone_t done,
@@ -647,7 +675,8 @@ dns_xfrin_create2(dns_zone_t *zone, dns_rdatatype_t xfrtype,
 
 isc_result_t
 dns_xfrin_create3(dns_zone_t *zone, dns_rdatatype_t xfrtype,
-		  isc_sockaddr_t *masteraddr, isc_sockaddr_t *sourceaddr,
+		  const isc_sockaddr_t *masteraddr,
+		  const isc_sockaddr_t *sourceaddr,
 		  isc_dscp_t dscp, dns_tsigkey_t *tsigkey, isc_mem_t *mctx,
 		  isc_timermgr_t *timermgr, isc_socketmgr_t *socketmgr,
 		  isc_task_t *task, dns_xfrindone_t done,
@@ -672,7 +701,8 @@ dns_xfrin_create3(dns_zone_t *zone, dns_rdatatype_t xfrtype,
 	CHECK(xfrin_start(xfr));
 
 	xfr->done = done;
-	xfr->refcount++;
+	if (xfr->done != NULL)
+		xfr->refcount++;
 	*xfrp = xfr;
 
  failure:
@@ -747,17 +777,17 @@ xfrin_reset(dns_xfrin_ctx_t *xfr) {
 
 	if (xfr->tcpmsg_valid) {
 		dns_tcpmsg_invalidate(&xfr->tcpmsg);
-		xfr->tcpmsg_valid = ISC_FALSE;
+		xfr->tcpmsg_valid = false;
 	}
 
 	if (xfr->ver != NULL)
-		dns_db_closeversion(xfr->db, &xfr->ver, ISC_FALSE);
+		dns_db_closeversion(xfr->db, &xfr->ver, false);
 }
 
 
 static void
 xfrin_fail(dns_xfrin_ctx_t *xfr, isc_result_t result, const char *msg) {
-	if (result != DNS_R_UPTODATE) {
+	if (result != DNS_R_UPTODATE && result != DNS_R_TOOMANYRECORDS) {
 		xfrin_log(xfr, ISC_LOG_ERROR, "%s: %s",
 			  msg, isc_result_totext(result));
 		if (xfr->is_ixfr)
@@ -774,7 +804,8 @@ xfrin_fail(dns_xfrin_ctx_t *xfr, isc_result_t result, const char *msg) {
 		(xfr->done)(xfr->zone, result);
 		xfr->done = NULL;
 	}
-	xfr->shuttingdown = ISC_TRUE;
+	xfr->shuttingdown = true;
+	xfr->shutdown_result = result;
 	maybe_free(xfr);
 }
 
@@ -788,15 +819,15 @@ xfrin_create(isc_mem_t *mctx,
 	     dns_name_t *zonename,
 	     dns_rdataclass_t rdclass,
 	     dns_rdatatype_t reqtype,
-	     isc_sockaddr_t *masteraddr,
-	     isc_sockaddr_t *sourceaddr,
+	     const isc_sockaddr_t *masteraddr,
+	     const isc_sockaddr_t *sourceaddr,
 	     isc_dscp_t dscp,
 	     dns_tsigkey_t *tsigkey,
 	     dns_xfrin_ctx_t **xfrp)
 {
 	dns_xfrin_ctx_t *xfr = NULL;
 	isc_result_t result;
-	isc_uint32_t tmp;
+	uint32_t tmp;
 
 	xfr = isc_mem_get(mctx, sizeof(*xfr));
 	if (xfr == NULL)
@@ -815,13 +846,14 @@ xfrin_create(isc_mem_t *mctx,
 	xfr->connects = 0;
 	xfr->sends = 0;
 	xfr->recvs = 0;
-	xfr->shuttingdown = ISC_FALSE;
+	xfr->shuttingdown = false;
+	xfr->shutdown_result = ISC_R_UNSET;
 
 	dns_name_init(&xfr->name, NULL);
 	xfr->rdclass = rdclass;
 	isc_random_get(&tmp);
-	xfr->checkid = ISC_TRUE;
-	xfr->id	= (isc_uint16_t)(tmp & 0xffff);
+	xfr->checkid = true;
+	xfr->id	= (uint16_t)(tmp & 0xffff);
 	xfr->reqtype = reqtype;
 	xfr->dscp = dscp;
 
@@ -830,7 +862,7 @@ xfrin_create(isc_mem_t *mctx,
 	/* qbuffer */
 	/* qbuffer_data */
 	/* tcpmsg */
-	xfr->tcpmsg_valid = ISC_FALSE;
+	xfr->tcpmsg_valid = false;
 
 	xfr->db = NULL;
 	if (db != NULL)
@@ -848,6 +880,7 @@ xfrin_create(isc_mem_t *mctx,
 	xfr->nmsg = 0;
 	xfr->nrecs = 0;
 	xfr->nbytes = 0;
+	xfr->maxrecords = dns_zone_getmaxrecords(zone);
 	isc_time_now(&xfr->start);
 
 	xfr->tsigkey = NULL;
@@ -856,7 +889,7 @@ xfrin_create(isc_mem_t *mctx,
 	xfr->lasttsig = NULL;
 	xfr->tsigctx = NULL;
 	xfr->sincetsig = 0;
-	xfr->is_ixfr = ISC_FALSE;
+	xfr->is_ixfr = false;
 
 	/* ixfr.request_serial */
 	/* ixfr.current_serial */
@@ -872,7 +905,7 @@ xfrin_create(isc_mem_t *mctx,
 	CHECK(dns_timer_setidle(xfr->timer,
 				dns_zone_getmaxxfrin(xfr->zone),
 				dns_zone_getidlein(xfr->zone),
-				ISC_FALSE));
+				false));
 
 	xfr->masteraddr = *masteraddr;
 
@@ -933,11 +966,11 @@ xfrin_start(dns_xfrin_ctx_t *xfr) {
 static isc_result_t
 render(dns_message_t *msg, isc_mem_t *mctx, isc_buffer_t *buf) {
 	dns_compress_t cctx;
-	isc_boolean_t cleanup_cctx = ISC_FALSE;
+	bool cleanup_cctx = false;
 	isc_result_t result;
 
 	CHECK(dns_compress_init(&cctx, -1, mctx));
-	cleanup_cctx = ISC_TRUE;
+	cleanup_cctx = true;
 	CHECK(dns_message_renderbegin(msg, &cctx, buf));
 	CHECK(dns_message_rendersection(msg, DNS_SECTION_QUESTION, 0));
 	CHECK(dns_message_rendersection(msg, DNS_SECTION_ANSWER, 0));
@@ -960,6 +993,8 @@ xfrin_connect_done(isc_task_t *task, isc_event_t *event) {
 	dns_xfrin_ctx_t *xfr = (dns_xfrin_ctx_t *) event->ev_arg;
 	isc_result_t result = cev->result;
 	char sourcetext[ISC_SOCKADDR_FORMATSIZE];
+	char signerbuf[DNS_NAME_FORMATSIZE];
+	const char *signer = "", *sep = "";
 	isc_sockaddr_t sockaddr;
 	dns_zonemgr_t * zmgr;
 	isc_time_t now;
@@ -992,12 +1027,22 @@ xfrin_connect_done(isc_task_t *task, isc_event_t *event) {
 	result = isc_socket_getsockname(xfr->socket, &sockaddr);
 	if (result == ISC_R_SUCCESS) {
 		isc_sockaddr_format(&sockaddr, sourcetext, sizeof(sourcetext));
-	} else
-		strcpy(sourcetext, "<UNKNOWN>");
-	xfrin_log(xfr, ISC_LOG_INFO, "connected using %s", sourcetext);
+	} else {
+		strlcpy(sourcetext, "<UNKNOWN>", sizeof(sourcetext));
+	}
+
+	if (xfr->tsigkey != NULL && xfr->tsigkey->key != NULL) {
+		dns_name_format(dst_key_name(xfr->tsigkey->key),
+				signerbuf, sizeof(signerbuf));
+		sep = " TSIG ";
+		signer = signerbuf;
+	}
+
+	xfrin_log(xfr, ISC_LOG_INFO, "connected using %s%s%s",
+		  sourcetext, sep, signer);
 
 	dns_tcpmsg_init(xfr->mctx, xfr->socket, &xfr->tcpmsg);
-	xfr->tcpmsg_valid = ISC_TRUE;
+	xfr->tcpmsg_valid = true;
 
 	CHECK(xfrin_send_request(xfr));
  failure:
@@ -1032,7 +1077,6 @@ tuple2msgname(dns_difftuple_t *tuple, dns_message_t *msg, dns_name_t **target)
 	ISC_LIST_APPEND(rdl->rdata, rdata, link);
 
 	CHECK(dns_message_gettemprdataset(msg, &rds));
-	dns_rdataset_init(rds);
 	CHECK(dns_rdatalist_tordataset(rdl, rds));
 
 	CHECK(dns_message_gettempname(msg, &name));
@@ -1085,7 +1129,6 @@ xfrin_send_request(dns_xfrin_ctx_t *xfr) {
 
 	/* Formulate the question and attach it to the question name. */
 	CHECK(dns_message_gettemprdataset(msg, &qrdataset));
-	dns_rdataset_init(qrdataset);
 	dns_rdataset_makequestion(qrdataset, xfr->rdclass, xfr->reqtype);
 	ISC_LIST_APPEND(qname->list, qrdataset, link);
 	qrdataset = NULL;
@@ -1111,7 +1154,7 @@ xfrin_send_request(dns_xfrin_ctx_t *xfr) {
 		CHECK(dns_db_getsoaserial(xfr->db, NULL,
 					  &xfr->ixfr.request_serial));
 
-	xfr->checkid = ISC_TRUE;
+	xfr->checkid = true;
 	xfr->id++;
 	xfr->nmsg = 0;
 	xfr->nrecs = 0;
@@ -1159,7 +1202,7 @@ xfrin_send_request(dns_xfrin_ctx_t *xfr) {
 	if (soatuple != NULL)
 		dns_difftuple_free(&soatuple);
 	if (ver != NULL)
-		dns_db_closeversion(xfr->db, &ver, ISC_FALSE);
+		dns_db_closeversion(xfr->db, &ver, false);
 	return (result);
 }
 
@@ -1196,7 +1239,7 @@ xfrin_recv_done(isc_task_t *task, isc_event_t *ev) {
 	dns_message_t *msg = NULL;
 	dns_name_t *name;
 	dns_tcpmsg_t *tcpmsg;
-	dns_name_t *tsigowner = NULL;
+	const dns_name_t *tsigowner = NULL;
 
 	REQUIRE(VALID_XFRIN(xfr));
 
@@ -1227,6 +1270,8 @@ xfrin_recv_done(isc_task_t *task, isc_event_t *ev) {
 	msg->tsigctx = xfr->tsigctx;
 	xfr->tsigctx = NULL;
 
+	dns_message_setclass(msg, xfr->rdclass);
+
 	if (xfr->nmsg > 0)
 		msg->tcp_continuation = 1;
 
@@ -1234,19 +1279,27 @@ xfrin_recv_done(isc_task_t *task, isc_event_t *ev) {
 				   DNS_MESSAGEPARSE_PRESERVEORDER);
 
 	if (result == ISC_R_SUCCESS)
-		dns_message_logpacket(msg, "received message:\n",
-				      DNS_LOGCATEGORY_XFER_IN,
-				      DNS_LOGMODULE_XFER_IN,
-				      ISC_LOG_DEBUG(10), xfr->mctx);
+		dns_message_logpacket2(msg, "received message from",
+				       &tcpmsg->address,
+				       DNS_LOGCATEGORY_XFER_IN,
+				       DNS_LOGMODULE_XFER_IN,
+				       ISC_LOG_DEBUG(10), xfr->mctx);
 	else
 		xfrin_log(xfr, ISC_LOG_DEBUG(10), "dns_message_parse: %s",
 			  dns_result_totext(result));
 
 	if (result != ISC_R_SUCCESS || msg->rcode != dns_rcode_noerror ||
+	    msg->opcode != dns_opcode_query ||msg->rdclass != xfr->rdclass ||
 	    (xfr->checkid && msg->id != xfr->id)) {
-		if (result == ISC_R_SUCCESS)
+		if (result == ISC_R_SUCCESS && msg->rcode != dns_rcode_noerror)
 			result = ISC_RESULTCLASS_DNSRCODE + msg->rcode; /*XXX*/
-		if (result == ISC_R_SUCCESS || result == DNS_R_NOERROR)
+		else if (result == ISC_R_SUCCESS &&
+			 msg->opcode != dns_opcode_query)
+			result = DNS_R_UNEXPECTEDOPCODE;
+		else if (result == ISC_R_SUCCESS &&
+			 msg->rdclass != xfr->rdclass)
+			result = DNS_R_BADCLASS;
+		else if (result == ISC_R_SUCCESS || result == DNS_R_NOERROR)
 			result = DNS_R_UNEXPECTEDID;
 		if (xfr->reqtype == dns_rdatatype_axfr ||
 		    xfr->reqtype == dns_rdatatype_soa)
@@ -1389,7 +1442,8 @@ xfrin_recv_done(isc_task_t *task, isc_event_t *ev) {
 		 * We should have no outstanding events at this
 		 * point, thus maybe_free() should succeed.
 		 */
-		xfr->shuttingdown = ISC_TRUE;
+		xfr->shuttingdown = true;
+		xfr->shutdown_result = ISC_R_SUCCESS;
 		maybe_free(xfr);
 		break;
 	default:
@@ -1426,8 +1480,9 @@ xfrin_timeout(isc_task_t *task, isc_event_t *event) {
 
 static void
 maybe_free(dns_xfrin_ctx_t *xfr) {
-	isc_uint64_t msecs;
-	isc_uint64_t persec;
+	uint64_t msecs;
+	uint64_t persec;
+	const char *result_str;
 
 	REQUIRE(VALID_XFRIN(xfr));
 
@@ -1436,6 +1491,16 @@ maybe_free(dns_xfrin_ctx_t *xfr) {
 	    xfr->recvs != 0)
 		return;
 
+	INSIST(! xfr->shuttingdown || xfr->shutdown_result != ISC_R_UNSET);
+
+	/* If we're called through dns_xfrin_detach() and are not
+	 * shutting down, we can't know what the transfer status is as
+	 * we are only called when the last reference is lost.
+	 */
+	result_str = (xfr->shuttingdown ?
+		      isc_result_totext(xfr->shutdown_result) : "unknown");
+	xfrin_log(xfr, ISC_LOG_INFO, "Transfer status: %s", result_str);
+
 	/*
 	 * Calculate the length of time the transfer took,
 	 * and print a log message with the bytes and rate.
@@ -1447,7 +1512,7 @@ maybe_free(dns_xfrin_ctx_t *xfr) {
 	persec = (xfr->nbytes * 1000) / msecs;
 	xfrin_log(xfr, ISC_LOG_INFO,
 		  "Transfer completed: %d messages, %d records, "
-		  "%" ISC_PRINT_QUADFORMAT "u bytes, "
+		  "%" PRIu64 " bytes, "
 		  "%u.%03u secs (%u bytes/sec)",
 		  xfr->nmsg, xfr->nrecs, xfr->nbytes,
 		  (unsigned int) (msecs / 1000), (unsigned int) (msecs % 1000),
@@ -1486,7 +1551,7 @@ maybe_free(dns_xfrin_ctx_t *xfr) {
 		dns_name_free(&xfr->name, xfr->mctx);
 
 	if (xfr->ver != NULL)
-		dns_db_closeversion(xfr->db, &xfr->ver, ISC_FALSE);
+		dns_db_closeversion(xfr->db, &xfr->ver, false);
 
 	if (xfr->db != NULL)
 		dns_db_detach(&xfr->db);
@@ -1502,7 +1567,7 @@ maybe_free(dns_xfrin_ctx_t *xfr) {
  * transfer of <zone> from <address>: <message>
  */
 static void
-xfrin_logv(int level, const char *zonetext, isc_sockaddr_t *masteraddr,
+xfrin_logv(int level, const char *zonetext, const isc_sockaddr_t *masteraddr,
 	   const char *fmt, va_list ap)
 {
 	char mastertext[ISC_SOCKADDR_FORMATSIZE];
@@ -1522,12 +1587,12 @@ xfrin_logv(int level, const char *zonetext, isc_sockaddr_t *masteraddr,
  */
 
 static void
-xfrin_log1(int level, const char *zonetext, isc_sockaddr_t *masteraddr,
+xfrin_log1(int level, const char *zonetext, const isc_sockaddr_t *masteraddr,
 	   const char *fmt, ...)
 {
 	va_list ap;
 
-	if (isc_log_wouldlog(dns_lctx, level) == ISC_FALSE)
+	if (isc_log_wouldlog(dns_lctx, level) == false)
 		return;
 
 	va_start(ap, fmt);
@@ -1545,7 +1610,7 @@ xfrin_log(dns_xfrin_ctx_t *xfr, int level, const char *fmt, ...)
 	va_list ap;
 	char zonetext[DNS_NAME_MAXTEXT+32];
 
-	if (isc_log_wouldlog(dns_lctx, level) == ISC_FALSE)
+	if (isc_log_wouldlog(dns_lctx, level) == false)
 		return;
 
 	dns_zone_name(xfr->zone, zonetext, sizeof(zonetext));
